# -*- coding: utf-8 -*-

#   This file is part of emesene.
#
#    Emesene is free software; you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation; either version 2 of the License, or
#    (at your option) any later version.
#
#    emesene is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with emesene; if not, write to the Free Software
#    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA

import gobject

import Conversation
import ConversationWindow

class ConversationManager(gobject.GObject):
    '''This class handle a collection of conversations'''

    __gsignals__ = {
        # conversation, window
        'new-conversation-ui' : (gobject.SIGNAL_RUN_LAST, gobject.TYPE_NONE, 
        (gobject.TYPE_PYOBJECT,gobject.TYPE_PYOBJECT)),
        # conversation, window
        'close-conversation-ui' : (gobject.SIGNAL_RUN_LAST, gobject.TYPE_NONE, 
        (gobject.TYPE_PYOBJECT,gobject.TYPE_PYOBJECT)),
        # send conversation, message
        'send-message' : (gobject.SIGNAL_RUN_LAST, gobject.TYPE_NONE, 
        (gobject.TYPE_PYOBJECT,gobject.TYPE_STRING)),
        # receive conversation, (message params)
        'receive-message' : (gobject.SIGNAL_RUN_LAST, gobject.TYPE_NONE, 
        (gobject.TYPE_PYOBJECT,gobject.TYPE_STRING,gobject.TYPE_STRING,gobject.TYPE_STRING,gobject.TYPE_STRING,gobject.TYPE_STRING)),
    }

    def __init__(self, controller):
        '''Contructor'''

        gobject.GObject.__init__(self)

        self.controller = controller
        self.theme = controller.theme
        self.config = controller.config

        #list of (ConversationWindow, Conversation) tuples
        self.conversations = []
        
        self.user = ''
        
        self.signals = []
        sap = self.signals.append
        sap(self.config.connect('change::showHeader', self.updateUI))
        sap(self.config.connect('change::showToolbar', self.updateUI))
        sap(self.config.connect('change::showAvatars', self.updateUI))
        sap(self.config.connect('change::showAvatarMine', self.updateUI))
        sap(self.config.connect('change::showAvatarOther', self.updateUI))
        sap(self.config.connect('change::showStatusBar', self.updateUI))
        sap(self.config.connect('change::showTabCloseButton', self.updateUI))
        sap(self.config.connect('change::showSendButton', self.updateUI))

        self.controller.connect('preferences-changed', self.updateUI)
    
    def handleLogin(self, user):
        '''handle a new user login (close conversations or reconnect them, etc.)'''
        if self.user == user:
            # same user as previous, enable all open conversations
            self.enableAll()
        else:
            # another user logged in, close open conversations
            self.user = user
            self.closeAll()
       
    def updateUI(self, *args):
        for window, conversation in self.conversations:
            conversation.ui.update()

    def getOpenConversation(self, mail, switchboard = None):
        '''return (ConversationWindow, conversation) for this contact, 
        or return None if there isn't a conversation open yet'''
        
        self.removeClosedWindows()
        for window, conversation in self.conversations:
            members = conversation.getMembers()
            if len(members) == 1 and members[0] == mail:
                return (window, conversation)
            elif len(members) > 1 and mail in members:
                # groupchat, check if it's with the same contacts
                if switchboard and len(switchboard.members) == len(members) and \
                    sorted(switchboard.members.keys()) == sorted(members):
                        return (window, conversation)
        return None
        
    def removeClosedWindows(self):
        '''remove conversations for closed windows'''
        for window, conversation in self.conversations[:]:
            if window.isClosed() or conversation.isClosed() or \
               (window.notOpen and conversation.getStatus() == 'closed'):
                self.emit('close-conversation-ui', conversation, window)
                self.conversations.remove((window, conversation))

    def openConversation(self, msnp, mail, weStarted, switchboard=None):
        '''opens a new conversation and a new window or tab'''
        
        # create new switchboard if needed
        if switchboard is None:
            switchboard = msnp.newSwitchboard()
            switchboard.invite(mail)

        conversation = Conversation.Conversation(self.controller, switchboard)

        # add a tab if we use tabs and if there is a window open already
        useTabs = not self.config.user['windows']
        if useTabs and len(self.conversations) > 0:
            # open a new tab
            window = self.conversations[0][0]
            window.openTab(conversation)
            if weStarted:
                window.present()
        else:
            # open a new window
            window = ConversationWindow.ConversationWindow(self.controller, conversation)

            # Gnome does not support calling iconify() before show() (?)
            # so we call it before and after, other WMs may support it...
            if not weStarted:
                window.iconify() 
            window.show()
            if not weStarted:
                window.iconify()
                
        conversation.setWindow(window)
        window.set_icon(conversation.getWindowIcon())
        self.conversations.append((window, conversation))
        self.emit('new-conversation-ui', conversation, window)
        
        return window, conversation
    
    def newConversation(self, msnp, mail, switchboard, weStarted):
        '''Open a new conversation, or open an existing window or tab
        if switchboard is None we create the switchboard'''
        
        window = conversation = None
        result = self.getOpenConversation(mail, switchboard)
        
        if result is not None: 
            # conversation open already
            window, conversation = result
            
            # set new switchboard
            if switchboard != None:
                conversation.setSwitchboard(switchboard)
            
            # show the current window and tab
            if weStarted:
                if not self.config.user['windows']:
                    window.showTab(window.tabs.page_num(conversation.ui))
                window.present()
        else:
            # open new conversation with this contact
            window, conversation = self.openConversation(msnp, mail, 
                                weStarted, switchboard)
            
        return window, conversation
      
    def do_send_message(self, conversation, message):
        '''Send message to conversation'''
        conversation.do_send_message(message)
        
    def do_receive_message(self, conversation, mail, nick, message, format, charset):
        '''Receive a message from'''
        conversation.do_receive_message(mail, nick, message, format, charset)

    def closeAll(self):
        '''close all the conversations'''

        for window, conversation in self.conversations:
            window.hide()

        self.conversations = []
    
    def disableAll(self):
        '''close conversations and disable text input of conversation windows'''
      
        for window, conversation in self.conversations:
            conversation.ui.setInputEnabled(False)
            conversation.switchboard.leaveChat()
        
    def enableAll(self):
        '''reconnect conversations and re-enable text input'''

        for window, conversation in self.conversations:
            conversation.reconnect()
            conversation.ui.setInputEnabled(True)



# -*- coding: utf-8 -*-

#   This file is part of emesene.
#
#    Emesene is free software; you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation; either version 2 of the License, or
#    (at your option) any later version.
#
#    Emesene is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with emesene; if not, write to the Free Software
#    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA

import gtk
import desktop
from Parser import Url

class UserMenu(gtk.Menu):
    '''This class represents the popup menu that is displayed when you right
    click an user on the userList'''

    def __init__(self , controller, user, group):
        gtk.Menu.__init__(self)

        self.controller = controller
        
        self.user = user
        self.group = group

        openConversationMenuItem = self.newImageMenuItem(
            _("_Open Conversation"), gtk.STOCK_OPEN)
        self.add(openConversationMenuItem)
        openConversationMenuItem.show()

        sendMailMenuItem = self.newImageMenuItem(_("Send _Mail"), None,
            None, controller.theme.getSmiley("(e)"))
        self.add(sendMailMenuItem)
        sendMailMenuItem.show()

        copyEmail = self.newImageMenuItem(_("Copy email"), gtk.STOCK_COPY)
        self.add(copyEmail)
        copyEmail.show()
            

        renameUserMenuItem = self.newImageMenuItem(_("_Set contact alias..."),
            gtk.STOCK_EDIT)
        self.add(renameUserMenuItem)
        renameUserMenuItem.show()
        
        profileMenuItem = self.newImageMenuItem(_("View profile"),
            gtk.STOCK_NETWORK)
        self.add(profileMenuItem)
        profileMenuItem.show()
        profileMenuItem.connect('activate', self.on_profile_activate)
        
        self.controller.emit("usermenu-item-add", self)
        
        parser = controller.unifiedParser
        sep = False

        for i in user.nick, user.personalMessage:
            urls = [str(x.url) for x in parser.parse(None, i)
                    if type(x) == Url]
            
            for url in urls:
                if len(url) > 20:
                    dispUrl = url[:20] + '...'
                else:
                    dispUrl = url
                
                PMurl = self.newImageMenuItem(dispUrl, gtk.STOCK_OPEN)
                PMurl.connect("activate", self.on_PMurl, url) 
                
                if not sep:
                    self.add(gtk.SeparatorMenuItem())
                    sep = True
                self.add(PMurl)
        
        self.add(gtk.SeparatorMenuItem())

        if not user.blocked:
            blockUserMenuItem = self.newImageMenuItem(_("_Block"),
                gtk.STOCK_STOP)
            self.add(blockUserMenuItem)
            blockUserMenuItem.show()
            blockUserMenuItem.connect("activate", self.on_block_user_activate)
        else:
            unblockUserMenuItem = self.newImageMenuItem(_("_Unblock"))
            self.add(unblockUserMenuItem)
            unblockUserMenuItem.show()
            unblockUserMenuItem.connect("activate",
                self.on_unblock_user_activate)

        deleteUserMenuItem = self.newImageMenuItem(_("_Remove contact"),
            gtk.STOCK_DELETE)
        self.add(deleteUserMenuItem)
        deleteUserMenuItem.show()

        self.add(gtk.SeparatorMenuItem())

        moveMenuItem = self.newImageMenuItem(_("M_ove to group"),
            gtk.STOCK_REDO)
        moveMenu = gtk.Menu()

        for i in self.controller.msn.getGroupNames():
            i = i.replace('_', '__') # don't use _ as mnemonic
            menuItem = self.newImageMenuItem (i)
            moveMenu.add(menuItem)
            menuItem.connect("activate" , self.moveToActivate, i)

        moveMenuItem.set_submenu(moveMenu)
        moveMenuItem.show_all()
        self.add(moveMenuItem)

        copyMenuItem = self.newImageMenuItem(_("_Copy to group"),
            gtk.STOCK_COPY)
        copyMenu = gtk.Menu()

        for i in self.controller.msn.getGroupNames():
            i = i.replace('_', '__')
            menuItem = self.newImageMenuItem (i)
            copyMenu.add(menuItem)
            menuItem.connect("activate", self.copyToActivate, i)

        copyMenuItem.set_submenu(copyMenu)
        copyMenuItem.show_all()
        self.add(copyMenuItem)

        removeFromGroupMenuItem = self.newImageMenuItem(
            _("R_emove from group"), gtk.STOCK_DELETE)
        self.add(removeFromGroupMenuItem)
        removeFromGroupMenuItem.show()
        removeFromGroupMenuItem.connect("activate", self.removeFromActivate)

        self.add(gtk.SeparatorMenuItem())

        addUserMenuItem = self.newImageMenuItem(_("_Add contact..."),
            gtk.STOCK_ADD)
        self.add(addUserMenuItem)
        addUserMenuItem.show()

        self.show_all()

        renameUserMenuItem.connect("activate", self.on_rename_user_activate)
        addUserMenuItem.connect("activate", self.on_add_user_activate)
        deleteUserMenuItem.connect("activate", self.on_delete_user_activate)
        openConversationMenuItem.connect("activate",
            self.on_open_conversation_activate)

        copyEmail.connect("activate", self.on_copy_email_activate)
        sendMailMenuItem.connect("activate" , self.on_send_mail_activate)


    def newImageMenuItem(self, label, stock=None, img=None, animation=None):
        mi = gtk.ImageMenuItem(_(label))

        if stock != None:
            mi.set_image(gtk.image_new_from_stock(stock, gtk.ICON_SIZE_MENU))
        elif img != None:
            image = gtk.Image()
            image.set_from_pixbuf(img)
            mi.set_image(image)
        elif animation != None:
            image = gtk.Image()
            image.set_from_animation(animation)
            mi.set_image(image)
        return mi

    def on_PMurl(self, widget, url):
        desktop.open(str(url))

    def moveToActivate(self, _menuItem, group):
        self.controller.contacts.move_to_group(self.user.email, 
            self.group.name, group)

    def copyToActivate(self, _menuItem, group):
        self.controller.contacts.add_to_group(self.user.email, group)

    def removeFromActivate(self, _menuItem):
        self.controller.contacts.remove_from_group(self.user.email, 
            self.group.name)
        
    def on_add_user_activate(self, *args):
        self.controller.addUserDialog()

    def on_delete_user_activate(self, *args):
        self.controller.contacts.remove(self.user.email)

    def on_rename_user_activate(self, *args):
        self.controller.contacts.set_alias_dialog(self.user.email)

    def on_block_user_activate(self, *args):
        self.controller.contacts.block(self.user.email)

    def on_unblock_user_activate(self, *args):
        self.controller.contacts.unblock(self.user.email)

    def on_space_user_activate(self, *args):
        self.controller.seeSpace(self.user.email)

    def on_profile_activate(self, *args):
        self.controller.seeProfile(self.user.email)

    def on_open_conversation_activate(self, *args):
        self.controller.newConversation(None, self.user.email, None, True)
    
    def on_copy_email_activate(self, *args):
        clipboard = gtk.clipboard_get(gtk.gdk.SELECTION_CLIPBOARD)
        clipboard.set_text(self.user.email)

    def on_send_mail_activate(self, *args):
        desktop.open("mailto:%s" % self.user.email)



# -*- coding: utf-8 -*-
'''a module to handle groups'''

#   This file is part of emesene.
#
#    Emesene is free software; you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation; either version 2 of the License, or
#    (at your option) any later version.
#
#    emesene is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with emesene; if not, write to the Free Software
#    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
import gettext

import stock
import Object

_ = gettext.gettext

class GroupManager(Object.Object):
    '''this class represent an abstract class that provide methods
    to interact with groups, implementing this class with the
    undeliying protocol library let the upper layer use the services
    of the protocol implementation independently of the api, in this
    way, the protocol library can be modified (or replaced) 
    without effect on the clients'''

    def __init__(self, dialog, protocol):
        '''initialize the object, dialog is a implementation
        of abstract.dialog, it's used to interact with the user'''
        Object.Object.__init__(self)
        self.dialog = dialog
        self.protocol = protocol
        self.groups = {}
        
        # emitted when some attribute of the group changes
        # the first argument is the group object, the second
        # is a string representing the attribute that changed
        # the third is the old value of the attr
        self.signal_add('group-changed', 3)
        self.signal_add('group-name-changed', 2)
        self.signal_add('group-added', 1)
        self.signal_add('group-removed', 1)
        # contact account (not the object), group 
        self.signal_add('contact-added-to-group', 2)
        # contact account (not the object), group 
        self.signal_add('contact-removed-from-group', 2)

    def register(self, group):
        '''add a group object to the list'''
        if not self.exists(group.name):
            self.groups[group.name] = group
        else:
            debug("group %s already in groups" % (group.name,))
    
    # utility methods
    def exists(self, group_name):
        '''check if the group is on self.groups, return True if exists'''
        if group_name in self.groups:
            return True
        else:
            return False

    # this are the base callbacks, you should inherit from this class
    # and make methods to be notified on this events and then call this
    # methods, they do all the work, you just need to addapt the parameters
    def _on_group_added(self, name, identifier=None, contacts=None):
        '''method called when a new group is created'''
        if self.exists(name):
            debug("group %s already in groups" % (name,))
        else:
            group = Group(name, identifier, contacts)
            self.groups[name] = group 
            self.signal_emit('group-added', group)

    def _on_group_removed(self, name):
        '''method called when a group is removed'''
        if self.exists(name):
            group = self.groups[name]
            del self.groups[name]
            self.signal_emit('group-removed', group)
        else:
            debug("group %s not in groups" % (name,))

    def _on_group_renamed(self, old_name, new_name):
        '''method called when the name of the group is changed'''
        if self.exists(old_name):
            group = self.groups[old_name]
            group.name = new_name
            self.groups[group.name] = group
            self.signal_emit('group-name-changed', group, old_name)
            self.signal_emit('group-changed', group, 'name', old_name)
        else:
            debug("group %s not in groups" % (old_name,))

    def _on_contact_added_to_group(self, account, group_name):
        '''callback called when an account is added to a group'''
        if self.exists(group_name):
            if account not in self.groups[group_name].contacts:
                self.groups[group_name].contacts.append(account)
                self.signal_emit('contact-added-to-group', 
                    account, self.groups[group_name])
            else:
                debug("account %s already in group %s" % (account, 
                    group_name))
        else:
            debug("group %s not in self.groups" % (group_name,))
            
    def _on_contact_removed_from_group(self, account, group_name):
        '''callback called when an account is removed from a group'''
        if self.exists(group_name):
            if account in self.groups[group_name].contacts:
                self.groups[group_name].contacts.remove(account)
                self.signal_emit('contact-removed-from-group', 
                    account, self.groups[group_name])
            else:
                debug("account %s not in group %s" % (account, 
                    group_name))
        else:
            debug("group %s not in self.groups" % (group_name,))

    # protocol actions on groups
    def add(self, name):
        '''add a group'''
        pass

    def rename(self, name, new_name):
        '''rename a group'''
        pass

    def remove(self, name):
        '''remove a group'''
        pass

    # dialog
    def add_dialog(self):
        '''show a dialog to add a group'''
        self.dialog.add_group(self.add_cb)

    def rename_dialog(self, old_name):
        '''show a dialog showing the actual name of a group
        and asking for the new one'''
        self.dialog.rename_group(old_name, self.rename_cb)

    def remove_dialog(self, name):
        '''ask for confirmation on group deletion, it can be used the method
        directly, but it's good to ask :P'''
        self.dialog.yes_no(_(
            _("Are you sure you want to delete the %s group?") % (name, )),
            self.remove_cb, name)

    # callbacks (the values are set to '' because when the window get closed
    # presing the x the values should not be sent)
    def add_cb(self, response, group_name=''):
        '''callback for the dialog.add_group method'''
        if response == stock.ACCEPT:
            if group_name:
                self.add(group_name)

    def rename_cb(self, response, old_name='', new_name=''):
        '''callback called by dialog.rename_group'''
        if response == stock.ACCEPT:
            if old_name == new_name:
                self.dialog.warning(_("Old and new name are the same"))
            elif new_name:
                self.rename(old_name, new_name)
            else:
                self.dialog.warning(_("new name not valid"))

    def remove_cb(self, response, group_name=''):
        '''callback for the dialog.yes_no method, asking for
        confirmation un group delete'''
        if response == stock.YES:
            self.remove(group_name)

class Group(object):
    '''a class representing a group'''

    def __init__(self, name, identifier=None, contacts=None):
        '''class constructor'''
        self.name = name
        self.identifier = identifier or '0'
        self.contacts = contacts or []

    def _on_contact_added(self, account):
        '''callback called when a contact is added to this group'''
        self.contacts.append(account)

    def _on_contact_removed(self, account):
        '''callback called when a contact is removed from this group'''
        if account in self.contacts:
            del self.contacts[account]

    def __repr__(self):
        '''return a string representation of the object'''
        return "<group name='%s'>" % (self.name,)

def debug(msg):
    '''debug method, the module send the debug here, it can be changed
    to use another debugging method'''
    return
    print('GroupManager.py: ', msg)
    



# -*- coding: utf-8 -*-

#   This file is part of emesene.
#
#    Emesene is free software; you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation; either version 2 of the License, or
#    (at your option) any later version.
#
#    emesene is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with emesene; if not, write to the Free Software
#    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA

VERSION = '0.3'

import Plugin
import gettext
import commands
import dialog
from emesenecommon import PATH

ERROR = ''

try:
    import gtkspell
except:
    ERROR = _('You need to install gtkspell to use Spell plugin')

class MainClass(Plugin.Plugin):
    '''Main plugin class'''
    
    def __init__(self, controller, msn):
        '''Constructor'''
        
        Plugin.Plugin.__init__(self, controller, msn)
        
        self.description = _('SpellCheck for emesene')
        self.authors = { 'Roger Duran' : 'RogerDuran at gmail dot com' }
        self.website = 'http://www.rogerpc.com.ar'
        self.displayName = _('Spell')
        self.name = 'Spell'
        
        self.controller = controller
        self.config = controller.config
        self.conversationManager = self.controller.conversationManager
                
        self.config.readPluginConfig(self.name)
        self.lang = self.config.getPluginValue(self.name, 'lang', '')
        
        self.newConversationWindowId = 0
        self.closeConversationWindowId = 0
        self.onInputFormatChangedId = 0
        
    def onInputFormatChanged(self, controller, textView):

        buffer = textView.get_buffer()
        if not buffer:
            return
        
        table = buffer.get_tag_table()
        if not table:
            return
        
        tag = table.lookup('gtkspell-misspelled')
        if not tag:
            return
        
        tag.set_priority(table.get_size() - 1)
    
    def start(self):
        '''start the plugin'''
        # this signals should be emitted by conversationManager
        self.newConversationWindowId = self.conversationManager.connect(
            'new-conversation-ui', self.newConversationWindow)
        self.closeConversationWindowId = self.conversationManager.connect(
            'close-conversation-ui', self.closeConversationWindow)
        self.onInputFormatChangedId = self.controller.connect(
            'input-format-changed', self.onInputFormatChanged)
        
        self.enabled = True
        
        self.applyAllConv(self.setSpell)
        self.applyAllConv(self.setLang)
        
    def stop(self, removeSpell=True):
        '''stop the plugin'''
        self.conversationManager.disconnect(self.newConversationWindowId)
        self.conversationManager.disconnect(self.closeConversationWindowId)
        self.controller.disconnect(self.onInputFormatChangedId)
        
        self.applyAllConv(self.removeSpell)
        
        self.enabled = False
        
    def error(self, message, removeSpell=True):
        if self.enabled:
           dialog.error( message + " " + \
               _("Plugin disabled."))
        self.stop(removeSpell)

    def applyAllConv(self, command):
        '''Applies a command to all open convs'''
        if self.enabled:
            for conversation in self.getOpenConversations():
                textView = conversation.ui.input.input
                if not command(textView):
                    return False
        
    def removeSpell(self, textView):
        try:
            gtkspell.get_from_text_view(textView).detach()
        except (SystemError, Exception):
            print "Can't detach gtkspell. Ignoring."
            return
        
    def setSpell(self, textView):
        try:
            gtkspell.Spell(textView, self.lang)
        except Exception, e:
            print str(e)
            self.error(_('Error applying Spell to input (%s)') % e, False)
            return False
        
    def setLang(self, textView):
        if self.lang == '':
            self.lang = None
        try:
            gtkspell.get_from_text_view(textView).set_language(self.lang)
        except Exception, e:
            print str(e)
            self.error(_('Error applying Spell to input (%s)') % e, False)
            return False
        
    def check(self):
        '''
        check if everything is OK to start the plugin
        return a tuple whith a boolean and a message
        if OK -> (True, 'some message')
        else -> (False, 'error message')
        '''
        if ERROR != '':
            return (False, ERROR)
            
        return (True, 'Ok')

    def newConversationWindow(self, conversationmanager, conversation, win):
        textView = conversation.ui.input.input  #inputwidget.textview
        self.setSpell(textView)
        
    def closeConversationWindow(self, conversationManager, conversation, win):
        textView = conversation.ui.input.input  #inputwidget.textview
        try:
            gtkspell.get_from_text_view(textView).detach()
        except:
            pass

    def configure(self):
        '''display a configuration dialog'''
        # name, optionType, label, description, value, options
        
        status, langs = commands.getstatusoutput('aspell dump dicts')
        if status == 0:
            langs = langs.split('\n')
        else:
            self.error(_("Error getting dictionaries list"))
            langs = []

        if not langs:
            self.error(_("No dictionaries found."))
            return
        
        l = []
        l.append(Plugin.Option('lang', list, _('Default language'),
            _('Set the default language'),
            self.config.getPluginValue(self.name, 'lang', ''), langs))
        
        response = Plugin.ConfigWindow(_('Spell configuration'), l).run()
        if response != None and response.has_key('lang'):
            self.config.setPluginValue(self.name, 'lang',
                str(response['lang'].value))
        
        self.lang = self.config.getPluginValue(self.name, 'lang', '')
        
        self.applyAllConv(self.setLang)
        
        return True



# -*- coding: utf-8 -*-

#   This file is part of emesene.
#
#    Emesene is free software; you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation; either version 2 of the License, or
#    (at your option) any later version.
#
#    emesene is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with emesene; if not, write to the Free Software
#    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA

import gc
import re
import gtk
import pango

import desktop
import Widgets
import FancyLabel
import StatusMenu
from SmileyRenderer import SmileyLabel
from Parser import PangoDataType

DOMAIN_REGEXP = re.compile('^live(\.[a-zA-Z]{2,3})?\.[a-zA-Z]{2,3}$|'
                           '^msn(\.[a-zA-Z]{2,3})?\.[a-zA-Z]{2,3}|'
                           '^hotmail(\.[a-zA-Z]{2,3})?\.[a-zA-Z]{2,3}')
class UserPanel(gtk.HBox):
    '''this class is the panel that will contain the user photo, his nick and his status'''
    
    def __init__(self, controller):
        '''Constructor'''        
        self.tooltip = gtk.Tooltips()
        
        self.controller = controller
        self.theme = controller.theme
        self.config = controller.config
                
        gtk.HBox.__init__(self)
        
        self.set_border_width(4)
        self.set_spacing(2)
        
        self.smileysCache = {}
        self.hasSmilies = True
            
        self.tNick = gtk.Entry(max=129)
        self.tNick.set_text(self.controller.msn.nick)
        self.tNick.set_no_show_all(True)

        if self.config.user['useFancyLabel']:
            self.lNick = FancyLabel.FancyLabel(self.controller)
            self.lNick.set_wrap(False) 
        else:
            self.lNick = SmileyLabel('', self.controller.widget_style)
            self.lNick.set_ellipsize(pango.ELLIPSIZE_END)

        self.bNick = gtk.Button()
        self.bNick.set_relief(gtk.RELIEF_NONE)
        self.bNick.set_alignment(0, 0)
        self.bNick.add(self.lNick)
        
        nick = self.controller.msn.nick

        if not self.config.user['useFancyLabel']:
            parser = self.controller.unifiedParser.getParser(nick, \
                PangoDataType)
            nick = parser.get(self.hasSmilies, self.smileysCache)
        
        self.lNick.set_markup(nick)
        
        iconMail = self.theme.getSmiley("(e)") #email icon
                
        self.mailButton = ImageButton(iconMail, '(0)')
        self.mailButton.set_relief(gtk.RELIEF_NONE)

        self.tPersonalMessage = gtk.Entry()
        self.tPersonalMessage.set_text(self.controller.contacts.get_message())
        self.tPersonalMessage.set_no_show_all(True)
        self.lPersonalMessage = SmileyLabel('', 
            self.controller.widget_style)
        self.lPersonalMessage.set_ellipsize(pango.ELLIPSIZE_END)
        self.bPersonalMessage = gtk.Button()
        self.bPersonalMessage.set_relief(gtk.RELIEF_NONE)
        self.bPersonalMessage.set_alignment(0, 0)
        self.bPersonalMessage.add(self.lPersonalMessage)
        
        pm = self.controller.contacts.get_message()
        if pm != '':
            parser = self.controller.unifiedParser.getParser(pm, PangoDataType)
            pm = parser.get(self.hasSmilies, self.smileysCache)
            self.lPersonalMessage.set_markup(pm)
        else:
            self.lPersonalMessage.set_text('<i>&lt;' + _('Click here to set your personal message') + '&gt;</i>')
                
        mediaIcon = self.theme.getSmiley("(8)") #media icon
        # at the bottom is the code to handle starting toggled
        self.mediaButton = ImageToggleButton(mediaIcon)
        self.mediaButton.set_relief(gtk.RELIEF_NONE)
        
        self.lMedia = gtk.Label(_("No media playing"))
        self.lMedia.set_ellipsize(pango.ELLIPSIZE_END)
        self.bMedia = gtk.Button()
        self.bMedia.set_relief(gtk.RELIEF_NONE)
        self.bMedia.set_alignment(0, 0)
        self.bMedia.add(self.lMedia)
        self.bMedia.connect('clicked', self.onMediaClicked)
        self.lMedia.set_no_show_all(True)
        self.bMedia.set_no_show_all(True)

        self.tooltip.set_tip(self.bNick, _('Click here to set your nick name'))
        self.tooltip.set_tip(self.bPersonalMessage, _('Click here to set your personal message'))
        self.tooltip.set_tip(self.bMedia, _('Your current media'))

        self.image = Widgets.avatarHolder(cellDimention = 48)
        self.imageEventBox = gtk.EventBox()
        self.imageEventBox.set_events(gtk.gdk.BUTTON_PRESS_MASK)
        self.imageEventBox.connect('button-press-event', self.avatarClicked)
        self.imageEventBox.add(self.image)
        #frame = gtk.AspectFrame()
        #frame.add(self.imageEventBox)
        if self.controller.avatar != None:
            self.image.set_from_pixbuf(self.controller.avatar.getThumb())
        else:
            self.pixbuf = self.controller.theme.getImage('userPanel')
            self.image.set_from_pixbuf(self.pixbuf)
        
        self.tNick.connect('activate', self.on_nick_changed)
        self.tNick.connect('focus-out-event', self.on_nick_changed)
        self.bNick.connect('clicked', self.on_nick_clicked)
        self.tPersonalMessage.connect('activate', self.on_pm_changed)
        self.tPersonalMessage.connect('focus-out-event', self.on_pm_changed)
        self.bPersonalMessage.connect('clicked', self.on_pm_clicked)
        self.mediaButton.connect("toggled", self.onToggleMedia)
        
        self.mailButton.connect('clicked', self.onMaiButtonClicked)

        self.controller.msn.connect('self-personal-message-changed', self.personalMessageChanged)
        self.controller.msn.connect('self-nick-changed', self.selfNickChanged)
        self.controller.msn.connect('self-current-media-changed', self.currentMediaChanged)
        
        self.mailButton.setText('('+str(self.controller.getUnreadMails()) +')')
        self.controller.msn.connect('initial-mail-notification', self.updateMailCount)
        self.controller.msn.connect('new-mail-notification', self.updateMailCount)
        self.controller.msn.connect('mail-movement-notification', self.updateMailCount)

        self.hbox = gtk.HBox()
        self.pack_start(self.imageEventBox, False, False)
        
        self.vbox1 = gtk.VBox()
        self.vbox2 = gtk.VBox(True)
        self.hbox1 = gtk.HBox()
        self.hbox2 = gtk.HBox()
        
        self.hbox1.pack_start(self.tNick, True, True)
        self.hbox1.pack_start(self.bNick, True, True)
        
        self.hbox2.pack_start(self.tPersonalMessage, True, True)
        self.hbox2.pack_start(self.bPersonalMessage, True, True)
        self.hbox2.pack_start(self.bMedia, True, True)
        
        self.vbox2h1 = gtk.HBox()
        self.vbox2h1.pack_start(self.mailButton, True, True)
        self.vbox2h2 = gtk.HBox()
        self.vbox2h2.pack_start(self.mediaButton, True, True)
        self.vbox2.pack_start(self.vbox2h1, True, False)
        self.vbox2.pack_start(self.vbox2h2, True, False)
        
        self.vbox1.pack_start(self.hbox1, True, False, 1)
        self.vbox1.pack_start(self.hbox2, True, False, 1)
        
        self.hbox.pack_start(self.vbox1, True, True)
        self.hbox.pack_start(self.vbox2, False, False)
        self.pack_start(self.hbox)
        
        self.show_all()
        
        # if in the config is active we activate it emitting the toggled signal
        if controller.config.user['mediaEnabled']:
            self.mediaButton.set_active(True)
        
        self.mediaButton.hide()
        
        try:
            if not DOMAIN_REGEXP.match(self.controller.userEmail.split("@")[1]):
                self.mailButton.hide()
        except Exception, e:
            print "error! " + str(e)
    
    def onMediaClicked(self, *args):
        self.controller.pluginManager.getPlugin('CurrentSong').configure()
        
    def avatarClicked(self, widget, event):
        if event.button == 1:
            self.controller.set_picture_dialog()
        elif event.button == 3:
            menu = AvatarMenu(self.controller)
            menu.popup(None, None, None, event.button, event.time)
        
    def updateMailCount(self, *args):
        self.mailButton.setText('('+str(self.controller.getUnreadMails()) +')')
            
    def selfNickChanged(self, msnp, oldNick, nick):
        '''method called when the user change his nick in other part'''
        self.tNick.set_text(nick)
        if not self.config.user['useFancyLabel']:
            parser = self.controller.unifiedParser.getParser(nick, \
                PangoDataType)
            nick = parser.get(self.hasSmilies, self.smileysCache)
        self.lNick.set_markup(nick)
        self.nickRefresh()

    def on_nick_activate(self, *args):
        self.controller.contacts.set_nick(self.tNick.get_text())
        
    def on_nick_changed(self, *args):
        self.controller.contacts.set_nick(self.tNick.get_text())

        self.tNick.hide()
        self.bNick.show()
        self.lNick.show()

    def on_nick_clicked(self, *args):
        self.tNick.show()
        self.bNick.hide()
        self.lNick.hide()
        self.tNick.grab_focus()
    
    def nickRefresh(self):
        self.on_nick_changed()
        self.bNick.grab_focus()

    def on_pm_changed(self, *args) : 
        self.controller.contacts.set_message(\
            self.tPersonalMessage.get_text())
            
        self.bMedia.hide()
        self.lMedia.hide()
        self.tPersonalMessage.hide()
        self.bPersonalMessage.show()
        self.lPersonalMessage.show()
               
    def on_pm_clicked(self, *args):
        self.bMedia.hide()
        self.lMedia.hide()
        self.tPersonalMessage.show()
        self.bPersonalMessage.hide()
        self.lPersonalMessage.hide()
        self.tPersonalMessage.grab_focus()
            
    def personalMessageRefresh(self):
        self.controller.contacts.set_message(\
            self.tPersonalMessage.get_text())

        if not self.mediaButton.get_active():
            self.bMedia.hide()
            self.lMedia.hide()
            self.tPersonalMessage.hide()
            self.bPersonalMessage.show()
            self.lPersonalMessage.show()
            self.bPersonalMessage.grab_focus()

    def personalMessageChanged(self, msnp, user, pm):
        '''method called when the pm is changed in other place'''
        
        self.tPersonalMessage.set_text(pm)
        if pm == '':
            self.lPersonalMessage.set_text('<i>&lt;' + _('Click here to set your personal message') + '&gt;</i>')
        else:
                parser = self.controller.unifiedParser.getParser(pm, PangoDataType)
                pm = parser.get(self.hasSmilies, self.smileysCache)
                self.lPersonalMessage.set_markup(pm)
            
    def currentMediaChanged(self, msnp, user, cm, dict):
        '''method called when the current media is changed in other place'''
        
        if cm != '':
            cm = cm[cm.find('\\0Music\\01\\0')+12:]
            cmargs = cm.split('\\0')
            cm = cmargs[0]
            for args in range(1, len(cmargs)):
                cm = cm.replace('{%s}' %str(args-1), cmargs[args])
                                
            self.lMedia.set_text('♫ ' + cm)
        else:
            self.lMedia.set_text(_("No media playing"))
                    
    def on_personal_activate(self, *args):
        self.controller.contacts.set_message(\
            self.tPersonalMessage.get_text())
        
    def nickChanged(self, nick):
        self.tNick.set_text(nick)
            
    def setAvatar(self, pixbuf):
        if pixbuf:
            self.image.set_from_pixbuf(pixbuf)
    
    def onToggleMedia(self, widget, *args):
        if widget.get_active():
           self.bPersonalMessage.hide()
           self.lPersonalMessage.hide()
           self.tPersonalMessage.hide()
           self.bMedia.show()
           self.lMedia.show()
           self.controller.setMediaEnabled(True)
        else:
            self.bMedia.hide()
            self.lMedia.show()
            self.tPersonalMessage.hide()
            self.bPersonalMessage.show()
            self.lPersonalMessage.show()
            
            self.controller.setMediaEnabled(False)
            
    def onMaiButtonClicked(self, *args):
        desktop.open(self.controller.hotmail.getLoginPage())
        
class BaseImageButton:
    def __init__(self, icon, string=None):
        self.icon = icon
        self.image = gtk.Image()
        hbox = gtk.HBox()
        self.setIcon(icon)
        hbox.pack_start(self.image, True, True, 3)
        
        if string != None:
            self.label = gtk.Label(string)
            hbox.pack_start(self.label, False, False, 3)
        
        self.add(hbox)
        
    def setText(self, string):
        self.label.set_text(string)
        
    def getText(self):
        return self.label.get_text()
        
    def setIcon(self, icon):
        if type(icon) == gtk.gdk.PixbufAnimation:
            self.image.set_from_pixbuf(self.scaleImage(icon.get_static_image()))
        elif type(icon) == gtk.gdk.Pixbuf:
            self.image.set_from_pixbuf(self.scaleImage(icon))
        else:
            self.image.set_from_stock(gtk.STOCK_MISSING_IMAGE ,gtk.ICON_SIZE_SMALL_TOOLBAR)
        
    def getIcon(self):
        return self.icon

    def scaleImage(self, image):
        h,w = image.get_height(), image.get_width()
        width_max, height_max = 18, 16
        width=float(image.get_width())
        height=float(image.get_height())
        if (width/width_max) > (height/height_max):
            height=int((height/width)*width_max)
            width=width_max
        else:
            width=int((width/height)*height_max)
            height=height_max

        image = image.scale_simple(width, height, gtk.gdk.INTERP_BILINEAR)
        gc.collect() # Tell Python to clean up the memory
        return image

class ImageButton(gtk.Button, BaseImageButton):
    def __init__(self, icon, string=None):
        gtk.Button.__init__(self)
        BaseImageButton.__init__(self, icon, string)
        
class ImageToggleButton(gtk.ToggleButton, BaseImageButton):
   def __init__(self, icon, string=None):
        gtk.ToggleButton.__init__(self)
        BaseImageButton.__init__(self, icon, string)

class AvatarMenu(gtk.Menu):
    '''This class represents the avatar menu, where you can change your
    status or avatar'''

    def __init__(self, controller):
        '''Contructor'''
        gtk.Menu.__init__(self)
        self.controller = controller
        
        statusMenuItem = gtk.MenuItem(_('_Status'))
        statusMenu = StatusMenu.StatusMenu(self.controller)
        statusMenuItem.set_submenu(statusMenu)
        
        changeAvatarMenuItem = gtk.MenuItem(_('Change _display picture...'))
        changeAvatarMenuItem.connect('activate', self.changeAvatar)
        
        self.add(statusMenuItem)
        self.add(gtk.SeparatorMenuItem())
        self.add(changeAvatarMenuItem)
        
        self.show_all()

    def changeAvatar(self, *args):
        '''Open avatar dialog'''
        self.controller.set_picture_dialog()

# the __main__ code went obsolete



# -*- coding: utf-8 -*-

#   This file is part of emesene.
#
#    Emesene is free software; you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation; either version 2 of the License, or
#    (at your option) any later version.
#
#    emesene is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with emesene; if not, write to the Free Software
#    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA

import xml.parsers.expat

class DynamicParser:
    '''Parse dynamic xml'''
    def __init__(self, xml_raw):
        '''init parser and setup handlers'''
        self.parser = xml.parsers.expat.ParserCreate()
        self.parser.buffer_text = True
        self.parser.returns_unicode = False
        
        self.groups = []
        self.contacts = []
        self.annotations = []
        self.group_ids = []
        
        self.in_group = False
        self.in_contact = False
        self.in_annotation = False
        self.in_group_ids = False
        
        self.group_data = {}
        self.contact_data = {}
        self.annotation_data = {}
        self.group_ids_data = {}
        
        self.current_tag = ''
        
        #connect handlers
        self.parser.StartElementHandler = self.start_element
        self.parser.EndElementHandler = self.end_element
        self.parser.CharacterDataHandler = self.char_data
        self.parser.Parse(xml_raw)
                
        del(xml_raw)
        
    def start_element(self, name, attrs):
        '''Start xml element handler'''
        if name == 'Group':
            self.in_group = True
        elif name == 'Contact':
            self.in_contact = True
        elif name == 'Annotation':
            self.in_annotation = True
        elif name == 'groupIds':
            self.in_group_ids = True
        self.current_tag = name
        
    def end_element(self, name):
        '''End xml element handler'''
        if name == 'Group':
            self.in_group = False
            if len(self.group_data) > 0:
                if len(self.annotations) > 0:
                    self.group_data.update({'Annotations':self.annotations})
                    self.annotations = []
                self.groups.append(self.group_data)
                self.group_data = {}
        elif name == 'Contact':
            self.in_contact = False
            if len(self.contact_data) > 0:
                annotations = self.annotations
                self.contact_data.update({'Annotations':annotations})
                self.contact_data.update({'groupIds':self.group_ids})
                self.contacts.append(self.contact_data)
                self.contact_data = {}
                self.annotations = []
                self.group_ids = []
        elif name == 'Annotation':
            self.in_annotation = False
            if len(self.annotation_data) > 0:
                self.annotations.append(self.annotation_data)
                self.annotation_data = {}
        elif name == 'groupIds':
            self.in_group_ids = False
            if len(self.group_ids_data) > 0:
                self.group_ids.append(self.group_ids_data)
                self.group_ids_data = {}

    def char_data(self, data):
        '''Char xml element handler'''
        if self.in_group_ids:
            self.group_ids.append(data)
        elif self.in_annotation:
            self.annotation_data.update({self.current_tag:data})
        elif self.in_group:
            self.group_data.update({self.current_tag:data})
        elif self.in_contact:
            self.contact_data.update({self.current_tag:data})

class MembershipParser:
    '''Parse membership xml'''
    def __init__(self, xml_raw):
        '''init parser and setup handlers'''
        self.parser = xml.parsers.expat.ParserCreate()
        self.parser.buffer_text = True
        self.parser.returns_unicode = False
        
        self.memberships = []
        self.members = []
        
        self.in_membership = False
        self.in_member = False
        
        self.membership_data = {}
        self.member_data = {}
        
        self.current_tag = ''
        
        #connect handlers
        self.parser.StartElementHandler = self.start_element
        self.parser.EndElementHandler = self.end_element
        self.parser.CharacterDataHandler = self.char_data
        self.parser.Parse(xml_raw)
        del(xml_raw)
        
    def start_element(self, name, attrs):
        '''Start xml element handler'''
        if name == 'Membership':
            self.in_membership = True
        elif name == 'Member':
            self.in_member = True
        self.current_tag = name
        
    def end_element(self, name):
        '''End xml element handler'''
        if name == 'Membership':
            self.in_membership = False
            if len(self.membership_data) > 0:
                self.membership_data.update({'Members':self.members})
                self.memberships.append(self.membership_data)
                self.membership_data = {}
                self.members = []
        if name == 'Member':
            self.in_member = False
            if len(self.member_data) > 0:
                self.members.append(self.member_data)
                self.member_data = {}

    def char_data(self, data):
        '''Char xml element handler'''
        if self.in_member:
            self.member_data.update({self.current_tag:data})
        elif self.in_membership:
            self.membership_data.update({self.current_tag:data})



# -*- coding: utf-8 -*-

#   This file is part of emesene.
#
#    Emesene is free software; you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation; either version 2 of the License, or
#    (at your option) any later version.
#
#    emesene is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with emesene; if not, write to the Free Software
#    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA

import pygtk
import gtk
import gtk.gdk
import cairo
import gobject

''' A set of useful cairo related methods '''    
def set_context_color(context, color = gtk.gdk.color_parse('#000000')):
    r = float(color.red) / 65535
    g = float(color.green) / 65535
    b = float(color.blue) / 65535
    context.set_source_rgb(r,g,b)
    
def image_surface_from_pixbuf( pixbuf=None):
    if pixbuf is not None:
        return cairo.ImageSurface.create_for_data(
            pixbuf.get_pixels_array(), 
            cairo.FORMAT_ARGB32,
            pixbuf.get_width(),
            pixbuf.get_height(), 
            pixbuf.get_rowstride())
    else:
        return cairo.ImageSurface(cairo.FORMAT_ARGB32, 1, 1)

def resize_image_surface(surface, new_width, new_height):
    new_surface = cairo.ImageSurface(cairo.FORMAT_ARGB32, new_width, new_height)
    cr = cairo.Context(new_surface)
    cr.set_source_surface(surface, 0, 0)
    cr.paint()
    return new_surface

class avatarHolder( gtk.Widget ):
    """avatarHolderWidget """
    
    #note: i should think up a better name than avatarHolder
    #cellKeyPosition, is a Anchor Type constant
    #refer to http://www.pygtk.org/docs/pygtk/gtk-constants.html#gtk-anchor-type-constants    
    __gproperties__ = {
        'pixbuf': (gtk.gdk.Pixbuf, 'Pixbuf',
        'Pixbuf',gobject.PARAM_READWRITE),
        'crossFade' : (bool,'animate by crossfade if true','',True, gobject.PARAM_READWRITE), 
        'dimention' : (gobject.TYPE_FLOAT,'cell dimensions',
        'height width of cell',0.0, 96.0,32.0, gobject.PARAM_READWRITE),
        'radius_factor' : (gobject.TYPE_FLOAT,'radius of pixbuf',
        '0.0 to 0.5 with 0.1 = 10% of dimention',0.0, 0.5,0.11, gobject.PARAM_READWRITE),
        'keyPositin' : (object,'position of avatar',
        'corresponds to position of key in numpad', gobject.PARAM_READWRITE),                        
        }             

    __gsignals__ = { 'size_request' : 'override', 'expose-event' : 'override' }
        
    def __init__(self, cellDimention = 96, crossFade = True, cellRadius = 0.05, cellKeyPosition = gtk.ANCHOR_CENTER):
        gobject.GObject.__init__(self)
        gtk.Widget.__init__(self)
        self.set_flags(self.flags() | gtk.NO_WINDOW )
        
        self._pixbuf = None
        self._dimention = cellDimention
        self._radius_factor = cellRadius
        self._keyPosition = cellKeyPosition
        
        
        # variables related to animation
        self._crossFade = crossFade               
        self.inAnimation = False
        self.duration = 1500    # milliseconds
        self.fps = 24           # frames per second
        self.totalFrames = 0
        self.currentFrame = 0               
        self.transitionPixbuf = None                              

    def do_get_property(self, property):
        if property.name == 'pixbuf':              
            return self._pixbuf
        elif property.name == 'dimention':
            return self._dimention
        elif property.name == 'radius-factor':
            return self._radius_factor   
        elif property.name == 'keyPosition':
            return self._keyPosition
        elif property.name == 'crossFade':
            return self._crossFade                                                                      
        else:
            raise AttributeError, 'unknown property %s' % property.name
  
    def do_set_property(self, property, value):
        if property.name == 'pixbuf':
            if self.__shouldReplace(value):
                if self._crossFade and not (self._pixbuf == None) and not (value == None) :                                
                    self.transitionPixbuf = value
                    if self.fps < 1: self.fps = 24 # reset fps if not valid fps
                    timeBetweenFrames = 1000 / self.fps
                    self.totalFrames = self.duration / timeBetweenFrames
                    self.currentFrame = 1
                    gobject.timeout_add( timeBetweenFrames, self.animate_callback)                               
                    self.inAnimation = True
                else:    
                    self._pixbuf = value
        elif property.name == 'dimention':
            self._dimention = value
        elif property.name == 'radius-factor':
            self._radius_factor = value
        elif property.name == 'keyPosition':
            self._keyPosition  = value
        elif property.name == 'crossFade':
            self._crossFade = value                                                  
        else:
            raise AttributeError, 'unknown property %s' % property.name
      
    def animate_callback(self):
        if self.currentFrame > self.totalFrames :
            self.inAnimation = False
            self._pixbuf = self.transitionPixbuf
            return False
        else:
            self.currentFrame += 1
            self.queue_draw()
            return True
                
        
    def set_from_pixbuf(self, pixbuf ):
        self.set_property('pixbuf', pixbuf)
        self.queue_draw()
    
    def set_from_file(self,filename):
        inputimage = open(filename)
        imagebuf = inputimage.read()
        try:
            pixbufloader = gtk.gdk.PixbufLoader()
            pixbufloader.write(imagebuf)
            pixbufloader.close()
        except:
            return
        pixbuf = pixbufloader.get_pixbuf()
        self.set_property('pixbuf', pixbuf)
        self.queue_draw()
                
    def do_size_request(self,requisition):
        requisition.width = self._dimention
        requisition.height = self._dimention

    def do_expose_event(self, evnt):
        if not self._pixbuf: return
               
        ctx =  evnt.window.cairo_create() 
        cell_area = self.get_allocation()               
                
        if self.inAnimation :
            self.__draw( ctx , cell_area , self._pixbuf, 1 - (float(self.currentFrame) / self.totalFrames))                        
            self.__draw( ctx , cell_area , self.transitionPixbuf,(float(self.currentFrame) / self.totalFrames))
        else:
            self.__draw( ctx , cell_area , self._pixbuf, 1)

    def __shouldReplace(self,pixbuf):
        #checkEquivalence
        if (self._pixbuf != None) and (pixbuf != None) and pixbuf.get_pixels() == self._pixbuf.get_pixels():         
            return False
        else: 
            return True
                            
    def __draw (self,ctx, cell_area, pixbuf, alpha):
        ctx.save()
        ctx.set_antialias(cairo.ANTIALIAS_SUBPIXEL)                                
        cell_x, cell_y, cell_width, cell_height = cell_area                                
        pix_width = pixbuf.get_width()
        pix_height = pixbuf.get_height()
        ctx.translate(cell_x, cell_y)      
                
        if (pix_width > self._dimention) or (pix_height > self._dimention): 
            scale_factor = float(self._dimention) / max (pix_width,pix_height)
        else: 
            scale_factor = 1
                
        scale_width = pix_width* scale_factor
        scale_height = pix_height* scale_factor
        
        self.__translate_keyPostion( ctx, self._keyPosition,cell_width,cell_height,scale_width,scale_height)
        
        scale_radius = self._dimention * self._radius_factor                
        self.__roundedrecMoonlight(ctx,0,0,scale_width,scale_height, scale_radius)
        
        ctx.clip()
        ctx.scale(scale_factor,scale_factor)
        ctx.set_source_pixbuf(pixbuf,0,0)        
        
        ctx.paint_with_alpha (alpha)
        
        ctx.restore()       

    def __translate_keyPostion(self,cr, key, w, h ,sw, sh):
        if key in [gtk.ANCHOR_NORTH_WEST,gtk.ANCHOR_WEST,gtk.ANCHOR_SOUTH_WEST] : x = 0
        elif key in [gtk.ANCHOR_NORTH,gtk.ANCHOR_CENTER,gtk.ANCHOR_SOUTH] : x = (w/2) - (sw/2)
        else: x = w - sw
        
        if key in [gtk.ANCHOR_NORTH_WEST,gtk.ANCHOR_NORTH,gtk.ANCHOR_NORTH_EAST] : y = 0
        elif key in [gtk.ANCHOR_EAST,gtk.ANCHOR_CENTER,gtk.ANCHOR_WEST] : y = (h/2) - (sh/2)
        else: y = h - sh 
        
        cr.translate( x, y)
        
    def __roundedrecMoonlight(self, cr,x,y,w,h,radius=5):
        # http://cairographics.org/cookbook/roundedrectangles/
        # modified from mono moonlight aka mono silverlight
        # test limits (without using multiplications)
        # http://graphics.stanford.edu/courses/cs248-98-fall/Final/q1.html

        ARC_TO_BEZIER = 0.55228475
        if radius > (min(w,h)/2):
            radius = (min(w,h)/2)
        #approximate (quite close) the arc using a bezier curve
        c = ARC_TO_BEZIER * radius

        cr.new_path();
        cr.move_to ( x + radius, y)
        cr.rel_line_to ( w - 2 * radius, 0.0)
        cr.rel_curve_to ( c, 0.0, radius, c, radius, radius)
        cr.rel_line_to ( 0, h - 2 * radius)
        cr.rel_curve_to ( 0.0, c, c - radius, radius, -radius, radius)
        cr.rel_line_to ( -w + 2 * radius, 0)
        cr.rel_curve_to ( -c, 0, -radius, -c, -radius, -radius)
        cr.rel_line_to (0, -h + 2 * radius)
        cr.rel_curve_to (0.0, -c, radius - c, -radius, radius, -radius)
        cr.close_path ()
                                             
gobject.type_register( avatarHolder )


class inputBox( gtk.TextView ):
    __gsignals__ = {
        'message-send':(gobject.SIGNAL_RUN_LAST|gobject.SIGNAL_ACTION,
                        gobject.TYPE_NONE, ()),
        'escape-pressed':(gobject.SIGNAL_RUN_LAST|gobject.SIGNAL_ACTION,
                          gobject.TYPE_NONE, ())
    }
    def __init__(self):
        gobject.GObject.__init__(self)
        gtk.binding_entry_add_signal(self, gtk.keysyms.KP_Enter, 0, 'message-send')
        gtk.binding_entry_add_signal(self, gtk.keysyms.Return, 0, 'message-send')
        gtk.binding_entry_add_signal(self, gtk.keysyms.Escape, 0, 'escape-pressed')
        
gobject.type_register( inputBox )


class TinyArrow(gtk.DrawingArea):
    LENGTH = 8
    WIDTH = 5

    def __init__(self, arrow_type, shadow=gtk.SHADOW_NONE):
        gtk.DrawingArea.__init__(self)
        self.arrow_type = arrow_type
        self.shadow = shadow
        self.margin = 0
        
        self.set_size_request(*self.get_size())
        self.connect("expose_event", self.expose)

    def get_size(self):
        if self.arrow_type in (gtk.ARROW_LEFT, gtk.ARROW_RIGHT):
            return (TinyArrow.WIDTH + self.margin*2, \
                    TinyArrow.LENGTH + self.margin*2)
        else:
            return (TinyArrow.LENGTH + self.margin*2, \
                    TinyArrow.WIDTH + self.margin*2)

    def expose(self, widget=None, event=None):
        if self.window is None:
            return
        self.window.clear()
        width, height = self.get_size()
        self.get_style().paint_arrow(self.window, self.state, \
            self.shadow, None, self, '', self.arrow_type, True, \
            0, 0, width, height)

        return False

    def set(self, arrow_type, shadow=gtk.SHADOW_NONE, margin=None):
        self.arrow_type = arrow_type
        self.shadow = shadow
        if margin is not None:
            self.margin = margin
        self.set_size_request(*self.get_size())
        self.expose()


class WidgetToggleBox(gtk.Widget):
    '''A box that represents a widget and allows toggling its visibility'''
    
    def __init__(self, config, key, description, label):
        gtk.Widget.__init__(self)
        self.description = description
        self.label = label
        self.config = config
        self.key = key
        self.enabled = False
        self.inside = False
        if self.config and key:
            self.enabled = self.config.user[key]

    def do_realize(self):
        '''Initializes the gtk window'''
        self.set_flags(gtk.REALIZED)

        mask = gtk.gdk.EXPOSURE_MASK | gtk.gdk.BUTTON_PRESS_MASK | \
               gtk.gdk.LEAVE_NOTIFY_MASK | gtk.gdk.ENTER_NOTIFY_MASK
        
        self.window = gtk.gdk.Window(
            self.get_parent_window(),
            x=self.allocation.x,
            y=self.allocation.y,
            width=self.allocation.width,
            height=self.allocation.height,
            window_type=gtk.gdk.WINDOW_CHILD,
            wclass=gtk.gdk.INPUT_OUTPUT,
            event_mask=self.get_events() | mask)

        self.window.set_user_data(self)
        self.style2 = self.style.copy()
        self.style2.attach(self.window)
        self.style2.set_background(self.window, self.state)
        self.window.move_resize(*self.allocation)

        if gtk.gtk_version >= (2, 12, 0) and \
           gtk.pygtk_version >= (2, 12, 0) and self.description:
            self.set_tooltip_text(self.description.replace("_", ""))

    def do_unrealize(self):
        '''Destroys the window'''
        self.window.set_user_data(None)
        self.window.destroy()

    def do_size_allocate(self, allocation):
        '''Resizes the window'''
        self.allocation = allocation
        if self.flags() & gtk.REALIZED:
            self.window.move_resize(*allocation)

    def do_expose_event(self, event):
        '''Renders the box'''
        if self.enabled:
            state = gtk.STATE_SELECTED
        else:
            if self.inside:
                state = gtk.STATE_PRELIGHT
            else:
                state = gtk.STATE_NORMAL

        self.style2.paint_box(self.window, state,
            gtk.SHADOW_ETCHED_IN, event.area, self, '', 0, 0,
            self.allocation.width, self.allocation.height)

    def do_button_press_event(self, event):
        '''Called when the user clicks the widget'''
        if self.key:
            self.enabled = not self.enabled
            self.queue_draw()
            if self.config:
                self.config.user[self.key] = self.enabled

    def do_enter_notify_event(self, event):
        '''Called when the mouse pointer enters the widget
        Sets a informative label text'''
        if self.description:
            self.inside = True
            self.label.set_text_with_mnemonic(self.description)
            self.queue_draw()

    def do_leave_notify_event(self, event):
        '''Called when the mouse pointer leaves the widget'''
        if self.description:
            self.inside = False
            self.label.set_text('')
            self.queue_draw()

gobject.type_register(WidgetToggleBox)



# -*- coding: utf-8 -*-
'''a module to handle contacts'''

#   This file is part of emesene.
#
#    Emesene is free software; you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation; either version 2 of the License, or
#    (at your option) any later version.
#
#    emesene is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with emesene; if not, write to the Free Software
#    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
import urllib

import abstract.ContactManager 
import status 

class ContactManager(abstract.ContactManager.ContactManager):
    '''Implementation of abstract.ContactManager using emesenelib'''

    def __init__(self, dialog, protocol, account):
        '''initialize the object, dialog is a implementation
        of abstract.dialog, it's used to interact with the user'''
        abstract.ContactManager.ContactManager.__init__(self, 
            dialog, protocol, account)
        
        # for dummy implementations
        if protocol:
            self._connect_signals()
        else:
            debug('Protocol is None')

    def _connect_signals(self):
        '''connect all the signals of the protocol'''
        # connect the signals to the private methods (see the double 
        # underscore, this methods receive the parameters and call
        # the methods of abstract.ContactManager.ContactManager
        self.protocol.connect('contact-added', self.__on_contact_added)
        self.protocol.connect('contact-removed', self.__on_contact_removed)

        self.protocol.connect('contact-attr-changed', 
            self.__on_contact_attr_changed)
        self.protocol.connect('nick-changed', 
            self.__on_contact_nick_changed)
        self.protocol.connect('contact-status-change', 
            self.__on_contact_status_changed)
        self.protocol.connect('initial-status-change', 
            self.__on_initial_status_changed)
        self.protocol.connect('personal-message-changed', 
            self.__on_contact_message_changed)
            
        self.protocol.connect('contact-added-to-group', 
            self.__on_contact_added_to_group)
        self.protocol.connect('contact-removed-from-group', 
            self.__on_contact_removed_from_group)

    def __on_contact_added(self, protocol, account, identifier=None, 
        nick=None, message=None, _status=status.OFFLINE, alias=None, 
        blocked=False):
        '''callback called when a new contact is added'''

        if _status in status.MSN_TO_STATUS:
            _status = status.MSN_TO_STATUS[_status]
        elif not status.is_valid(_status):
            debug("Invalid status '%s', setting offline" % (_status,))
            _status = status.OFFLINE

        self._on_contact_added(account, identifier, nick, message, _status,
            alias, blocked)

    def __on_contact_removed(self, protocol, account):
        '''callback called when a contact is removed'''
        self._on_contact_removed(account)

    def __on_contact_attr_changed(self, protocol, account, attr, val):
        '''callback called when an attribute is changed on a contact,
        this attribute can be block or alias (since these are the
        attributes that the user can change, the others are changed
        by the user itself)'''
        self._on_contact_attr_changed(account, attr, val)

    def __on_contact_nick_changed(self, protocol, account, nick):
        '''callback called when an user change his nick'''
        self._on_contact_nick_changed(account, nick)
    
    def __on_contact_message_changed(self, protocol, account, message):
        '''callback called when an user change his message'''
        self._on_contact_message_changed(account, message)

    def __on_contact_status_changed(self, protocol, account, new_status):
        '''callback called when an user change his status'''
        if new_status in status.MSN_TO_STATUS:
            self._on_contact_status_changed(account, 
                status.MSN_TO_STATUS[new_status])
        else:
            debug("invalid status '%s'" % (new_status,))

    def __on_initial_status_changed(self, protocol, command, tid, params):
        '''callback called when an user change his status'''
        (_status, account, nick, rest) = params.split(' ', 3)
        nick = urllib.unquote(nick)
        
        if self.exists(account):
            contact = self.contacts[account]
            old_nick = contact.nick
            old_status = contact.status
            contact._on_nick_changed(nick)
            contact._on_status_changed(status.MSN_TO_STATUS[_status])
            self.signal_emit('contact-changed', contact, 'nick', old_nick)
            self.signal_emit('contact-changed', contact, 'status', old_status)
            self.signal_emit('contact-nick-changed', contact, old_nick)
            self.signal_emit('contact-status-changed', contact, old_status)
        else:
            debug("account '%s' not in self.contacts" % (account,))


    def __on_contact_added_to_group(self, protocol, account, group_name):
        '''callback called when an account is added to a group'''
        self._on_contact_added_to_group(account, group_name)

    def __on_contact_removed_from_group(self, protocol, account, group_name):
        '''callback called when an account is removed from a group'''
        self._on_contact_removed_from_group(account, group_name)

    # actions on our contact
    def set_nick(self, nick):
        '''set the nick of our account to nick'''
        if nick:
            if self.protocol:
                self.me.nick = nick
                self.protocol.changeNick(nick)
        else:
            debug("invalid '%s' nick" % (nick,))

    def set_message(self, message):
        '''set the personal message of our account to message'''
        if self.protocol:
            self.me.message = message
            self.protocol.changePersonalMessage(message)

    def set_media(self, media):
        '''set the current media of out account to current_media'''
        if self.protocol:
            self.me.attrs['media'] = media
            self.protocol.changeCurrentMedia(media)

    def set_status(self, new_status):
        '''set the status to status, the status should be one of the
        constants on status.py, consider calling status.is_valid.
        Also you should convert it to the values on the library'''
        if not self.protocol:
            return

        if status.is_valid(new_status):                        # direct
            self.me.status = status.STATUS_TO_MSN[new_status]
            self.protocol.changeStatus(status.STATUS_TO_MSN[new_status])      
        elif new_status in status.STATUS_TO_MSN.values():             # HDN
            self.me.status = new_status
            self.protocol.changeStatus(new_status)
        else:
            debug('not a valid status"' + new_status + '"')

    def get_status(self, account=None):
        '''return the status of an account if exist, status.OFFLINE if dont
        if account == None, return the status of our user'''
        if account:
            if self.exists(account):
                return status.STATUS_TO_MSN[self.contacts[account].status]
            else:
                debug("account '%s' not in self.contacts" % (account,))
        else:
            return self.me.status

    def set_picture(self, path):
        '''set the display picture to path'''
        if self.protocol:
            self.protocol.changeAvatar(path)
            self.me.picture = path

    # actions on other contacts
    def set_alias(self, account, alias):
        '''set the contact alias, give an empty alias to reset'''
        if self.protocol:
            self.protocol.changeNick(alias, account)
            self._on_contact_attr_changed(account, 'alias', alias)

    def block(self, account):
        '''unblock an user'''
        if self.protocol:
            self.protocol.blockUser(account)
            self._on_contact_attr_changed(account, 'block', True)

    def unblock(self, account):
        '''unblock an user'''
        if self.protocol:
            self.protocol.unblockUser(account)
            self._on_contact_attr_changed(account, 'block', False)

    def remove(self, account):
        '''remove an user'''
        if self.protocol:
            self.protocol.removeUser(account) 

    def add(self, account, group=''):
        '''add an user'''
        if self.protocol:
            self.protocol.addUser(account, group)

    def move_to_group(self, account, src_group, dest_group):
        '''move a user from src_group to dest_group'''
        if self.protocol:
            self.protocol.moveUserToGroup(account, src_group, dest_group)

    def add_to_group(self, account, group):
        '''add an user to a group, return True on success'''
        if self.protocol:
            self.protocol.addUserToGroup(account, group)
    
    def remove_from_group(self, account, group):
        '''remove an user from a group'''
        if self.protocol:
            self.protocol.removeUserFromGroup(account, group)

def debug(msg):
    '''debug method, the module send the debug here, it can be changed
    to use another debugging method'''
    return
    print('ContactManager.py: ', msg)



# -*- coding: utf-8 -*-

#   This file is part of emesene.
#
#    Emesene is free software; you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation; either version 2 of the License, or
#    (at your option) any later version.
#
#    emesene is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with emesene; if not, write to the Free Software
#    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA

from os import name
if name == 'posix':
    from Amarok import Amarok
    from Exaile import Exaile
    from Xmms import Xmms
    from Banshee import Banshee
    from Audacious import Audacious
    from Rhythmbox import Rhythmbox
    from QuodLibet import QuodLibet
    from Listen import Listen
else:
    from Winamp import Winamp
from CurrentSong import CurrentSong
from Mpd import Mpd

del name

# dont put stuff in the global namespace



# -*- coding: utf-8 -*-

#   This file is part of emesene.
#
#    Emesene is free software; you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation; either version 2 of the License, or
#    (at your option) any later version.
#
#    emesene is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with emesene; if not, write to the Free Software
#    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA

import urllib
import base64
import sha
import os

class MsnObjectsManager(object):
    def __init__( self, user ):
        self.user = user
        self.objects = {}
        self.idToContext = {}
    
    def getByContext( self, context ):
        tmp = base64.b64decode( context )
        if not tmp.endswith( '\0' ):
            context = base64.b64encode( tmp+'\0' )
        if self.objects.has_key(context):
            return self.objects[context]
        else:
            return None
        
    def getIds( self ):
        return self.idToContext.keys()
        
    def getById( self, id ):
        return self.getByContext( self.idToContext[id] )
        
    def create( self, id, filename, type ):
        #msnobj = Msnobj.Msnobj( self.user, size, Msnobj.Msnobj.CUSTOM_EMOTICON, name, data = f.read() )
        f = file( filename, 'r' )
        f.seek( 0,2 )
        size = f.tell()
        f.seek( 0,0 )
        name = filename.split( os.sep )[ -1: ][ 0 ]
        msnobj = Msnobj( self.user.lower(), size, type, "0", data = f.read(), filename=filename, friendly=id )
        f.close()
        context = base64.b64encode( str( msnobj )+ '\0' )
        self.objects.update( {context:msnobj} )
        if id !='': self.idToContext.update( {id:context} )
        return msnobj
        
    def remove( self, id=None, context=None ):
        try:
            if id != None:
                context = self.getById( id )
                self.idToContext.pop( id )
            self.objects.pop( context )
        except: pass
                
class Msnobj( object ):
    '''this class represent the msnobject, for example:
    <msnobj Creator="buddy@hotmail.com" Size="24539" Type="3" Location="TFR2C.tmp" Friendly="AAA=" SHA1D="snip" SHA1C="snip"/>
    wlm CE:
    spoo    <msnobj Creator="buddy@hotmail.com" Type="2" SHA1D="87YzhijLQyyp4iIlL2k9LlB2wr0=" Size="1169" Location="0" Friendly="cwBwAG8AbwAAAA=="/>
    *  2: Custom Emoticons
    * 3: Display Pictures
    * 5: Background Images
    * 7: Dynamic Display Pictures (Flash-animated DPs)
    * 8: Winks (short Flash animations)
    * 11: Voice Clips (Wave sounds)
    * 12: "SavedState" Property of Add-Ins
    * 14: MSNP15 Locations 
    '''
    
    CUSTOM_EMOTICON = 2
    DISPLAY_PICTURE = 3
    BACKGROUND_IMAGE = 4
    DYNAMIC_DISPLAY_PICTURE = 7
    WINK = 8
    VOICE_CLIP = 11
    SAVED_STATE_PROPERTY = 12
    LOCATION = 14
    
    def __init__( self, creator, size='', type='', location='', friendly='\x00', sha1c='', sha1d='', data='', filename='' ):
        '''#  Creator - This field indicates the person who made (and is sending) the object.
            # Size - The Size field contains the length of the file in bytes
            # Type - The Type field indicates what sort of image (or file) it is about. At the moment the following flags are known:

            * 2: Custom Emoticons
            * 3: Display Pictures
            * 5: Background Images
            * 7: Dynamic Display Pictures (Flash-animated DPs)
            * 8: Winks (short Flash animations)
            * 11: Voice Clips (Wave sounds)
            * 12: "SavedState" Property of Add-Ins
            * 14: MSNP15 Locations 

            # Location - The Location field contains the filename under which the picture (or file) will be (or has been) stored
            # Friendly - This field contains the name of the picture in Unicode (UTF-16) format. The string is then encoded with Base64. So far this field has only been seen used with Background Images and Winks. With other file types, its value is "AAA=" (the utf-16 null string, encoded with Base64).
            # SHA1D - The SHA1D field contains a SHA1 hash of the images data encoded in Base64. It will from now on be called the SHA1 Data field.
            # SHA1C - This field contains all previous fields hashed with SHA1, and then encoded in Base64'''
        #FIXME: that docstring sucks
        self.creator = creator
        self.size = size
        self.type = int(type)
        self.location = location
        self.friendly = base64.b64encode( unicode( friendly ).encode( 'utf-16' )[2:] )
        self.filename = filename

        self.data = data
        #for received msnobj
        
        if sha1d != '':
            self.sha1d = sha1d
            # always return the same sha1d, amsn requires this
            self.forceSha1d = True
        else:
            self.sha1d = self.makeSha1d() 
            self.forceSha1d = False
        
        if sha1c != '':
            self.sha1c = sha1c
        else:
            self.sha1c = self.makeSha1c()
        
    def __repr__( self ):
        string = '<msnobj Creator="' + self.creator + '" '
        string += 'Size="' + str( self.size ) + '" Type="' + str( self.type ) + '" '
        string += 'Location="'+ self.location +'" '
        string += 'Friendly="' + self.friendly + '" '
        
        sha1d = ''
        if self.forceSha1d:
            sha1d = self.sha1d
        else:
            sha1d = self.makeSha1d()
        
        string += 'SHA1D="' + sha1d + '" SHA1C="' + self.makeSha1c() + '"/>'
        return string
    
    def makeSha1d( self ):
        return base64.b64encode( sha.new( self.data ).digest() )
    
    def makeSha1c( self ):
        string = 'Creator' + self.creator
        string += 'Size' + str( self.size ) + 'Type' + str( self.type )
        string += 'Location'+ unquote( self.location )
        string += 'Friendly' + self.friendly
        string += 'SHA1D' + self.sha1d
        
        return base64.b64encode( sha.new( string ).digest() )
    
    def quote( self ):
        return urllib.quote( self.__repr__() )
    
    def getCreator( self ): #XXX: property
        return self.creator
    
    def setCreator( self, creator ):
        self.creator = creator
        
    def getType(self):
        return str( self.type )

    def __eq__(self, obj):
        status = True
        if not isinstance(obj, Msnobj):
            return False

        fields = ['creator', 'location', 'friendly', 'size', 'type',
                  'sha1c', 'sha1d']

        for field in fields:
            if getattr(self, field) != getattr(obj, field):
                #print "DEBUG msnobj %s differ: %s != %s" % (field,
                #    getattr(self, field),
                #    getattr(obj, field))
                status = False
        return status

    def __neq__(self, obj):
        return not (self == obj)
    
            
def unquote( msnobj ):    
    return urllib.unquote( msnobj )

def createFromString( string, quoted=True ):
    if string == '0': return None
    
    if quoted:
        msnobjStr = unquote( string )
    else:
        msnobjStr = string
    
    def getData( string, start, stop ):
        if start in string:
            return string.split( start )[1].split(stop)[0]
        else:
            # TODO: what to do when a field is not in the string?
            return ''
    
    try:
        creator = getData( msnobjStr, 'Creator="', '"' )
        size = getData( msnobjStr, 'Size="', '"' )
        _type = getData( msnobjStr, 'Type="', '"' )
        location = getData( msnobjStr, 'Location="', '"' )
        friendly = getData( msnobjStr, 'Friendly="', '"' )
        try:
            friendly = str( base64.b64decode( friendly ).decode( 'utf-16' ) )
        except:
            friendly = '\x00'
        sha1d = getData( msnobjStr, 'SHA1D="', '"' )
        sha1c = getData( msnobjStr, 'SHA1C="', '"' )
        
        return Msnobj(creator, size, _type, location, friendly, sha1c, sha1d)
    except Exception, e:
        print 'cant create msnobj from: ' + msnobjStr
        print str( e )
        return None




# -*- coding: utf-8 -*-

#   This file is part of emesene.
#
#    Emesene is free software; you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation; either version 2 of the License, or
#    (at your option) any later version.
#
#    emesene is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with emesene; if not, write to the Free Software
#    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
'''a gtk implementation of abstract.Menu'''
import gtk

import stock
from abstract.Menu import *

accel_group = gtk.AccelGroup()

def build(element):
    '''build an element with the defined toolkit and return it'''
    if type(element) is Menu:
        return _build_menu(element)
    elif type(element) is Item:
        return _build_menuitem(element)
    elif type(element) is Option:
        return _build_option(element)
    elif type(element) is CheckBox:
        return _build_checkbox(element)

    raise ValueError('Not valid type', type(element), Menu)

def build_menu_bar(menu):
    '''build a menu with the form of a menu bar'''
    gtk_menubar = gtk.MenuBar()

    for child in menu:
        gtk_menubar.add(build(child))
    
    menu.signal_connect("enabled", _set_sensitive, gtk_menubar, True)
    menu.signal_connect("disabled", _set_sensitive, gtk_menubar, False)

    return gtk_menubar

def build_pop_up(menu, x=0, y=0):
    '''build a pop up menu, if positions are given and needed by the toolkit
    will be used to show the pop up on that position'''
    gtk_menu = gtk.Menu()

    for child in menu:
        item = build(child)
        item.show()
        gtk_menu.add(item)
    
    menu.signal_connect("enabled", _set_sensitive, gtk_menu, True)
    menu.signal_connect("disabled", _set_sensitive, gtk_menu, False)

    return gtk_menu

def _set_sensitive(item, widget, enabled):
    '''set sensitive on widget to enabled'''
    widget.set_sensitive(enabled)

def _build_menu(menu):
    '''build a gtk.Menu from a abstract.Menu.Menu'''

    gtk_menu = gtk.Menu()
    gtk_menu_item = gtk.MenuItem(menu.text)
    gtk_menu_item.set_submenu(gtk_menu)

    for item in menu:
        if type(item) is Item:
            if item.text == '-':
                gtk_menu.add(gtk.SeparatorMenuItem())
            else:
                gtk_menu.add(_build_menuitem(item))
        elif type(item) is Option:
            for option in _build_option(item):
                gtk_menu.add(option)
        else:
            gtk_menu.add(build(item))

    menu.signal_connect("enabled", _set_sensitive, gtk_menu_item, True)
    menu.signal_connect("disabled", _set_sensitive, gtk_menu_item, False)
            
    return gtk_menu_item

def _build_option(option):
    '''build an option object, return a list of gtk.RadioMenuItems'''
    option_list = []
    gtk_option = gtk.RadioMenuItem(None, option[0].text)

    if option.selected_index <= 0 or option.selected_index >= len(option):
        gtk_option.set_active(True)

    gtk_option.connect('activate', _option_cb, option[0], option)
    option_list.append(gtk_option)

    for (count, opt) in enumerate(option[1:]):
        gtk_opt = gtk.RadioMenuItem(gtk_option, opt.text)
        option.signal_connect("enabled", _set_sensitive, gtk_opt, True)
        option.signal_connect("disabled", _set_sensitive, gtk_opt, False)

        if option.selected_index == count + 1:
            gtk_opt.set_active(True)

        gtk_opt.connect('activate', _option_cb, opt, option)
        option_list.append(gtk_opt)
    
    option.signal_connect("enabled", _set_sensitive, gtk_option, True)
    option.signal_connect("disabled", _set_sensitive, gtk_option, False)

    return option_list

def _build_checkbox(checkbox):
    '''build a checkbox and return it'''
    gtk_checkbox = gtk.CheckMenuItem(checkbox.text)
    gtk_checkbox.set_active(checkbox.toggled)
    gtk_checkbox.connect('toggled', _checkbox_cb, checkbox)
    
    checkbox.signal_connect("enabled", _set_sensitive, gtk_checkbox, True)
    checkbox.signal_connect("disabled", _set_sensitive, gtk_checkbox, False)

    return gtk_checkbox

def _build_menuitem(menuitem):
    '''build a menu item from a abstract.Menu.Item'''
    gtk_menuitem = gtk.ImageMenuItem(menuitem.text) 

    if menuitem.is_stock():
        image = gtk.image_new_from_stock(
            stock.map_stock(menuitem.image, gtk.STOCK_MISSING_IMAGE),
            gtk.ICON_SIZE_MENU)
        gtk_menuitem.set_image(image)
    elif menuitem.is_image_path():
        image = gtk.image_new_from_file(menuite.get_image_path(), 
            gtk.ICON_SIZE_MENU)
        gtk_menuitem.set_image(image)

    if menuitem.accel:
        special_keys = 0

        if menuitem.accel.ctrl:
            special_keys = special_keys | gtk.gdk.CONTROL_MASK
        if menuitem.accel.alt:
            special_keys = special_keys | gtk.gdk.MOD1_MASK

        gtk_menuitem.add_accelerator('activate', accel_group, 
            ord(menuitem.accel.key), special_keys, gtk.ACCEL_VISIBLE)

    gtk_menuitem.connect('activate', _menu_item_cb, menuitem)

    menuitem.signal_connect("enabled", _set_sensitive, 
        gtk_menuitem, True)
    menuitem.signal_connect("disabled", _set_sensitive, 
        gtk_menuitem, False)

    return gtk_menuitem

def _menu_item_cb(gtk_mitem, mitem):
    '''callback called when a Item is selected, emit the selected signal
    on Item'''
    mitem.signal_emit('selected')

def _option_cb(gtk_option, option, options):
    '''callback selected when an option is toggled'''
    option.toggled = gtk_option.get_active()
    option.signal_emit('toggled', option.toggled)

def _checkbox_cb(gtk_checkbox, checkbox):
    '''callback called when the checbox is toggled'''
    checkbox.toggled = gtk_checkbox.get_active()
    checkbox.signal_emit('toggled', checkbox.toggled)

def test():
    '''test the implementation'''
    import dialog
    import abstract.MainMenu as MainMenu

    def quit_cb(item):
        '''method called when the quit item is selected'''
        gtk.main_quit()

    def status_cb(item, stat):
        '''method called when a status is selected'''
        print 'status', status.STATUS[stat], 'selected'

    window = gtk.Window()
    window.set_default_size(200, 200)
    window.connect('delete-event', gtk.main_quit)
    vbox = gtk.VBox()

    logged_in = True

    menu = MainMenu.MainMenu(dialog, None, None, None)
    vbox.pack_start(build_menu_bar(menu), False)
    vbox.pack_start(gtk.Label('test'), True, True)
    vbox.pack_start(gtk.Statusbar(), False)
    window.add(vbox)
    window.show_all()
    menu.status_item.enabled = False
    menu.order_option.enabled = False
    menu.quit_item.enabled = False
    menu.help_menu.enabled = False
    menu.show_by_nick_option.enabled = False
    gtk.main()

if __name__ == '__main__':
    test()



# -*- coding: utf-8 -*-

#   This file is part of emesene.
#
#    Emesene is free software; you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation; either version 2 of the License, or
#    (at your option) any later version.
#
#    emesene is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with emesene; if not, write to the Free Software
#    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA

import os
import gtk

if os.name == 'nt':
    import winsound
    
import Plugin
from emesenecommon import PATH

try: 
    import gst
    GSTREAMER = True
except:
    GSTREAMER = False
   
class Sound:
    '''A plugin to play sounds using the available modules on the system'''

    def __init__(self, theme):
        '''class constructor'''
    
        self.theme = theme
        self.beep = False
        self.command = ''
        self.canPlay = False
        self.canGstreamer = False
        
        if os.name == "posix":
            self.checkAvailability()
            if self.canGstreamer:
                self.player = gst.element_factory_make("playbin", "player")
                bus = self.player.get_bus()
                bus.enable_sync_message_emission()
                bus.add_signal_watch()
                bus.connect('message', self.gst_on_message)
        else:
            self.canPlay = True

    def gst_on_message(self, bus, message):
        t = message.type
        if t == gst.MESSAGE_EOS:
            self.player.set_state(gst.STATE_NULL)

    def checkAvailability(self):
        if self.beep:
            self.canPlay = True
        elif GSTREAMER:
            self.canPlay = True
            self.canGstreamer = True
        elif self.is_on_path('aplay'):
            self.canPlay = True
            self.command = 'aplay'
        elif self.is_on_path('play'):
            self.canPlay = True
            self.command = 'play'
        
    def play(self, sound_theme,  sound):
        if self.beep:
            gtk.gdk.beep()
            return
        
        for theme in (sound_theme, 'default'):
            soundPath = PATH.SOUNDS_PATH + os.sep + sound_theme + os.sep + \
                sound + ".wav"
            
            if os.path.exists(soundPath):
                break
            else:
                soundPath = ''

        if not soundPath:
            return
        
        if os.name == "nt":
            winsound.PlaySound(soundPath, 
                winsound.SND_FILENAME | winsound.SND_ASYNC)
        elif os.name == "posix":
            if self.canGstreamer:
                loc = "file://" + soundPath
                self.player.set_property('uri', loc)
                self.player.set_state(gst.STATE_PLAYING)    
            else:
                os.popen4(self.command + " " + soundPath)
            
    def getCommand(self):
        return self.command
        
    def setCommand(self, string):
        self.command = string
        
    def is_on_path(self, fname):
        for p in os.environ['PATH'].split(os.pathsep):
            if os.path.isfile(os.path.join(p, fname)):
                return True

class MainClass(Plugin.Plugin):
    '''Main plugin class'''
    
    def __init__(self, controller, msn):
        '''Contructor'''
        Plugin.Plugin.__init__(self, controller, msn)
        self.theme = controller.theme
        
        self.description = _('Play sounds for common events.')
        self.authors = { 'Mark Baas' : 'mark.baas123 at gmail dot com' }
        self.website = 'http://www.emesene.org'
        self.displayName = _('Sound')
        self.name = 'Sound'
        
        self.sound = Sound(self.theme)
        
        self.config = controller.config
        self.config.readPluginConfig(self.name)
        self.playOnline = int(self.config.getPluginValue(self.name, 
            'playOnline', '1'))
        self.playMessage = int(self.config.getPluginValue(self.name, 
            'playMessage', '1') )
        self.playNudge = int(self.config.getPluginValue(self.name, 
            'playNudge', '1'))
        self.playInactive = int(self.config.getPluginValue(self.name, 
            'playInactive', '1'))
        self.playSend = int(self.config.getPluginValue(self.name, 
            'playSend', '0'))
        self.disableBusy = int(self.config.getPluginValue(self.name, 
            'disableBusy', '0'))
        
        self.sound_theme = self.config.getPluginValue(self.name, 
            'theme', 'default')
        self.sound.beep = int(self.config.getPluginValue(self.name, 
            'beep', '0'))
        
        self.onlineId = None
        self.messageId = None
        self.nudgeId = None

    def start(self):
        '''start the plugin'''

        self.enabled = True
        self.onlineId = self.msn.connect('user-online', self.online)
        self.messageId = self.msn.connect('message-received', self.message)
        self.nudgeId = self.msn.connect('nudge-received', self.nudge)
        self.sendMessageId = self.controller.conversationManager.connect(
            'send-message', self.send)
        
    def stop(self):    
        '''stop the plugin'''
        self.msn.disconnect(self.onlineId)
        self.msn.disconnect(self.messageId)
        self.msn.disconnect(self.nudgeId)
        self.msn.disconnect(self.sendMessageId)
        
        self.enabled = False
        
    def action(self):
        pass        
    
    def check(self):
        '''
        check if everything is OK to start the plugin
        return a tuple whith a boolean and a message
        if OK -> (True, 'some message')
        else -> (False, 'error message')
        '''
        
        if not self.sound.canPlay:
            return (False, _('gstreamer, play and aplay not found.'))
        
        return (True, 'Ok')

    def online(self, msnp, email, oldStatus):
        self.playOnline = int(self.config.getPluginValue(self.name, 
            'playOnline', '1'))
        self.sound_theme = self.config.getPluginValue(self.name, 'theme', 
            'default')
        if oldStatus == 'FLN' and self.playOnline and self.soundsEnabled():
            self.sound.play(self.sound_theme, 'online')

    def message(self, msnp, email):
        self.playMessage = int(self.config.getPluginValue(self.name, 
            'playMessage', '1') )
        self.sound_theme = self.config.getPluginValue(self.name, 'theme', 
            'default')
        
        if self.playMessage and self.soundsEnabled():
            result = self.controller.conversationManager\
                .getOpenConversation(email)
            if self.playInactive and result != None:
                window, conversation = result
                windowFocus = window.is_active()
                tabFocus = (window.conversation == conversation)
                
                if not (windowFocus and tabFocus):
                    self.sound.play(self.sound_theme, 'type')
            else:
                self.sound.play(self.sound_theme, 'type')

    def nudge(self, *args):
        self.playNudge = int(self.config.getPluginValue(self.name, 
            'playNudge', '1'))
        self.sound_theme = self.config.getPluginValue(self.name, 
            'theme', 'default')
        if self.playNudge and self.soundsEnabled():
            self.sound.play(self.sound_theme, 'nudge')
    
    def send(self, *args):
        self.playSend = int(self.config.getPluginValue(self.name, 
            'playSend', '0'))
        self.sound_theme = self.config.getPluginValue(self.name, 
            'theme', 'default')
        if self.playSend and self.soundsEnabled():
            self.sound.play(self.sound_theme, 'send')
            
    def soundsEnabled(self):
        '''checks if sounds are enabled'''
        if not self.enabled:
            return False
        if self.disableBusy and self.controller.contacts.get_status() == 'BSY':
            return False
        return True
            
    def configure(self):
        '''display a configuration dialog'''
        l = []

        #sound theme
        themes = os.listdir(PATH.APP_PATH + os.sep + 'sound_themes')
        themes = [x for x in themes if not x.startswith('.')]

        # name, optionType, label, description, value, options
        l.append(Plugin.Option('theme', list, _('Theme'), '', 
            self.config.getPluginValue(self.name, 'theme', ''), themes))

        l.append(Plugin.Option('playOnline', bool, 
            _('Play online sound'), 
            _('Play a sound when someone gets online'), 
            (self.config.getPluginValue(self.name, 'playOnline', '1') \
                == '1')))
        
        l.append(Plugin.Option('playMessage', bool, 
            _('Play message sound'), 
            _('Play a sound when someone sends you a message'), 
            (self.config.getPluginValue(self.name,  'playMessage', '1') \
                == '1')))
        
        l.append(Plugin.Option('playNudge', bool, 
            _('Play nudge sound'), 
            _('Play a sound when someone sends you a nudge'), 
            (self.config.getPluginValue(self.name, 'playNudge', '1') \
                == '1')))
        
        l.append(Plugin.Option('playSend', bool, 
            _('Play sound when you send a message'), 
            _('Play sound when you send a message'), 
            (self.config.getPluginValue(self.name, 'playSend', '0') \
                == '1')))
        
        l.append(Plugin.Option('playInactive', bool, 
            _('Only play message sound when window is inactive'), 
            _('Play the message sound only when the window is inactive'), 
            (self.config.getPluginValue(self.name, 'playInactive', '1') \
                == '1')))
        
        l.append(Plugin.Option('disableBusy', bool, 
            _('Disable sounds when busy'), 
            _('Disable sounds when busy'), 
            (self.config.getPluginValue(self.name, 'disableBusy', '1') \
                == '1')))
        
        l.append(Plugin.Option('beep', bool, 
            _('Use system beep'), 
            _('Play the system beep instead of sound files'), 
            (self.config.getPluginValue(self.name, 'beep', '0') \
                == '1')))
        
        response = Plugin.ConfigWindow(_('Config Sound Plugin'), l).run()
        if response != None:
            if response.has_key('playOnline'):
                self.config.setPluginValue(self.name, 'playOnline', 
                    str(int(response['playOnline'].value)))
            if response.has_key('playMessage'):
                self.config.setPluginValue(self.name, 'playMessage', 
                    str(int(response['playMessage'].value)))
            if response.has_key('playNudge'):
                self.config.setPluginValue(self.name, 'playNudge', 
                    str(int(response['playNudge'].value)))
            if response.has_key('playInactive'):
                self.config.setPluginValue(self.name, 'playInactive', 
                    str(int(response['playInactive'].value)))
            if response.has_key('playSend'):
                self.config.setPluginValue(self.name, 'playSend', 
                    str(int(response['playSend'].value)))
            if response.has_key('beep'):
                self.config.setPluginValue(self.name, 'beep', 
                    str(int(response['beep'].value)))
            if response.has_key('theme'):
                self.config.setPluginValue(self.name, 'theme', 
                    response['theme'].value)
            if response.has_key('disableBusy'):
                self.config.setPluginValue(self.name, 'disableBusy', 
                    str(int(response['disableBusy'].value)))

        self.playOnline = (self.config.getPluginValue(self.name, 
            'playOnline', '1') == '1')
        self.playMessage = (self.config.getPluginValue(self.name, 
            'playMessage', '1') == '1')
        self.playNudge = (self.config.getPluginValue(self.name, 
            'playNudge', '1') == '1')
        self.playInactive = (self.config.getPluginValue(self.name, 
            'playInactive', '1') == '1')
        self.disableBusy = (self.config.getPluginValue(self.name, 
            'disableBusy', '1') == '1')
        self.sound.beep = int(self.config.getPluginValue(self.name, 
            'beep', '0'))
        
        return True
        



#!/usr/bin/python

from distutils.core import setup
import glob, os.path

from emesenecommon import APP_VERSION

# From apport's setup.py
mo_files = []
for filepath in glob.glob("po/*/LC_MESSAGES/*.mo"):
    lang = filepath[len("po/"):]
    targetpath = os.path.dirname(os.path.join("share/locale",lang))
    mo_files.append((targetpath, [filepath]))


setup(name         = 'emesene',
      version      = APP_VERSION,
      description  = 'MSN messenger client',
      author       = 'Luis Mariano Guerra',
      author_email = 'marianoguerra@users.sourceforge.net',
      url          = 'http://www.emesene.org/',
      license      = 'GNU GPL 2',
      requires     = ["gtk"],
      platforms    = ["Platform Independent"],
      packages     = ['', 'abstract', 'emesenelib', 'plugins_base', 'plugins_base.currentSong'],
      package_data = {'': ['conversation_themes/*/*', 'smilies/*/*',
                           'sound_themes/*/*', 'themes/*/*', 'hotmlog.htm']},
      scripts      = ['emesene'],
      data_files   = [('share/pixmaps', ['misc/emesene.png']),
                      ('share/icons/hicolor/scalable/apps', ['misc/emesene.svg']),
                      ('share/man/man1', ['misc/emesene.1']),
                      ('share/applications', ['misc/emesene.desktop'])] + mo_files
      )



# -*- coding: utf-8 -*-

#   This file is part of emesene.
#
#    Emesene is free software; you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation; either version 2 of the License, or
#    (at your option) any later version.
#
#    emesene is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with emesene; if not, write to the Free Software
#    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
import os
import gtk
import socket
import urllib
import urllib2
import gobject

import Plugin
import currentSong

from emesenecommon import PATH

AMAZON_KEY = '12DR2PGAQT303YTEWP02'
SEARCH_URL = 'http://webservices.amazon.com/onca/xml?' + \
    'Service=AWSECommerceService&AWSAccessKeyId=' + AMAZON_KEY + \
    '&Operation=ItemSearch&SearchIndex=Music&Artist=%s&' + \
    'ResponseGroup=Images&Keywords=%s'

COVERART_PATH = PATH.HOME_DIR + PATH.DIR_SEP + '.covers' + PATH.DIR_SEP 

class MainClass( Plugin.Plugin ):
    '''Main plugin class'''
    
    def __init__( self, controller, msn ):
        '''Constructor'''
        
        Plugin.Plugin.__init__( self, controller, msn, 1000 )
        self.theme = controller.theme
        
        self.controller = controller
        
        self.description = _( 'Show a personal message with the current song' \
            ' played in multiple players.' )
        self.authors = { 'Mariano Guerra' : 'luismarianoguerra gmail com' }
        self.website = 'http://emesene.org'
        self.displayName = _( 'CurrentSong' )
        self.name = 'CurrentSong'
        
        self.configWindow = None
        self.statusLabel = None
        self.customConfigWidget = None
        self.usingDummy = True
        self.dictCommand = None
        
        self.Slash = controller.Slash
        self.config = controller.config
        self.config.readPluginConfig(self.name)
        
        self.oldavatar = None
        
        self.setPlayer( self.config.getPluginValue( self.name, 'player', \
            'CurrentSong' ) )
        self.player.setStyle( self.config.getPluginValue( self.name, 'style', \
            '%title - %artist (%album)' ) )
        self.avatar = self.config.getPluginValue(self.name, 'avatar', \
            '0') == '1'
        self.remotefetch = self.config.getPluginValue(self.name, \
            'remotefetch', '0') == '1'
        
        self.updateCustomConf()
        self.timeout = 0
        self.mediaEnabledId = 0
        self.coverAvatar = Plugin.Function( self.getCoverAvatar, \
            self.setCoverAvatar )

    def start( self ):
        '''start the plugin'''
        if not os.path.isdir(COVERART_PATH):
            os.mkdir(COVERART_PATH)
        
        if not self.usingDummy:
            self.player.start()
        
        self.timeout = gobject.timeout_add(self.period, self.action)
        self.Slash.register('currentsong', self.currentSongHandler, \
            'currentsong command')
        self.Slash.register('np', self.nowplaying, \
            'Send what you are listening to')
        
        self.controller.mainWindow.userPanel.mediaButton.show()

        self.mediaEnabledId = self.controller.config.connect(
            'change::mediaEnabled', self.onMediaEnabled)
            
        self.enabled = True
    
    def showHelp(self):
        msg = 'Help:'
        for (cmd, val) in self.dictCommand.iteritems():
            msg += '\n ' + str(cmd) + '\t' + str(val[1])
        return msg

    def playerChanged( self, combo ):
        newplayer = combo.get_active_text()
        self.setPlayer(newplayer)
        self.updateStatus()
        self.updateCustomConf() 
        self.dictCommand = self.player.dictCommand
        self.customConfigWidget.refresh(self.player.customConfig)

    def currentSongHandler(self, slashAction):
        name = None
        param = None
        
        try:    
            cmd = slashAction.getParams().split(" ", 1)
            name = cmd[0]
            param = cmd[1]
        except:
            pass
        
        if name != None and self.dictCommand.has_key(name):
            retval, msg = self.dictCommand[name][0](param)
            if self.dictCommand[name][2]:
                slashAction.outputText( msg , retval )
            else:
                slashAction.outputText( msg )
        else:
            slashAction.outputText(self.showHelp())
    
    def nowplaying(self, slashAction):
        if self.player.cmd_Show(True)[0] == False:
            slashAction.outputText('Not playing')
        else:
            slashAction.outputText('now playing: '+ \
                self.player.cmd_Show(True)[1], True)

    def stop( self ):    
        '''stop the plugin'''
        
        if not self.usingDummy:
            self.player.stop()
        
        if self.timeout:
            gobject.source_remove(self.timeout)

        self.msn.changeCurrentMedia( '' )
        self.Slash.unregister('currentsong')
        self.Slash.unregister('np')
        
        self.controller.mainWindow.userPanel.mediaButton.hide()
        self.controller.mainWindow.userPanel.mediaButton.set_active(False)
        
        if self.oldavatar:
            self.controller.changeAvatar(self.oldavatar)
        self.oldavatar = None
        self.controller.config.disconnect(self.mediaEnabledId)
        
        self.enabled = False

    def onMediaEnabled(self, config, value, oldvalue):
        '''Called when the media button is toggled'''
        print "onMediaEnabled"
        self.action(True)
        
    def action(self, force=False):
        if not self.enabled or not self.config.user['mediaEnabled']:
            return True
        
        if not self.usingDummy:
            try:
                check = self.player.check()
            except Exception, e:
                print 'Error in player.check():', e
                self.setPlayer(None)
                check = True
            
            # USEFULL for LAST.FM plug 
            # self.controller.emit('media-changed',self.player.getSongDict())
            # dx note: wtf?

            if force or check: 
                currentsong = self.player.getCurrentSong()\
                    .decode("utf-8", "replace").encode("utf-8")
                sdict = self.player.getSongDict()
                
                self.msn.changeCurrentMedia( currentsong, sdict)
                if self.avatar:
                    self.coverAvatar(self.remotefetch)
        else:
            self.setPlayer( self.config.getPluginValue( self.name, 'player', \
                'CurrentSong' ) )

        return True

    def getCoverAvatar(self, remote):
        ''' get avatar cover art '''
        path = self.player.getCoverPath()
        if path and os.path.exists(path):
            return path

        path = self.search_local_image()  
        if path and os.path.exists(path):
            return path

        if remote:
            try:
                return self.search_remote_image()
            except:
                # handle connection errors
                pass

    def setCoverAvatar(self, path):
        ''' set avatar cover art '''
        if path != None and os.path.exists(path):
            self.oldavatar = self.config.user['avatarPath']
            self.controller.changeAvatar(path)
            return True
        if not self.player.isPlaying() or not self.player.isRunning():
            if self.oldavatar:
                self.controller.changeAvatar(self.oldavatar)
            self.oldavatar = None
            return False
        return False

    def is_on_path(self, fname):
        for p in os.environ['PATH'].split(os.pathsep):
            if os.path.isfile(os.path.join(p, fname)):
                return True
        
    def getPluginNames( self ):
        return [ x for x in dir( currentSong ) \
                 if not x.startswith( "__" ) \
                 and x != 'CurrentSong' ]
        
    def setPlayer( self, name ):
        '''try to set the player object to the name given by the name parameter
        if not set the dummy player to have something to play with'''
        
        players = self.getPluginNames()
        self.playername = name

        error = '' 
        if name in players:
            try:
                self.player = getattr(currentSong, name)()
                self.usingDummy = False
                self.action()
            except Exception, e:
                print "error initializing CurrentSong player:", e
                error = e
            else:
                status, message = self.player.getStatus()
                self.player.log(status and 'info' or 'error', \
                    'getStatus: ' + message)

                if not status:
                    self.player.status = 'error'
                
                self.dictCommand = self.player.dictCommand
                return
        self.player = currentSong.CurrentSong()
        self.usingDummy = True
        if error:
            return error
        
    def check( self ):
        '''check if everything is OK to start the plugin
        return a tuple whith a boolean and a message
        if OK -> ( True , 'some message' )
        else -> ( False , 'error message' )'''
        
        return (True, 'Ok')
   
    def updateCustomConf( self ):
        for key in self.player.customConfig.keys():
            self.player.customConfig[key] = \
                self.config.getPluginValue( self.name, \
                str(self.playername + "_" + key), \
                self.player.customConfig[key] )     

        self.player.updateConfig()

    def search_local_image(self):
        '''searches in the local covers cache
        returns None if no local image found'''
        
        artist = urllib.quote(self.player.artist.encode('utf8'))
        album = urllib.quote(self.player.album.encode('utf8'))
        file_dsc = COVERART_PATH + artist + '-' + album + '.jpg'
        if os.path.exists(file_dsc):
            return file_dsc
        return None

    def search_remote_image(self):
        '''searches in amazon for covers
        returns None if no images found'''
        imgfound = None
        
        artist = urllib.quote(self.player.artist.encode('utf8'))
        album = urllib.quote(self.player.album.encode('utf8'))
        if len(artist) == 0 or len(album) == 0:
            return imgfound
        dest_filename = COVERART_PATH + artist + '-' + album + '.jpg'
        
        HSIZE = 200 #header size -- magic number, FIXME
        img_url = ''
        
        for params in [(artist, album), (artist, ''), ('', album)]:
            try:
                request = urllib2.Request(SEARCH_URL % params)
                opener = urllib2.build_opener()
                f = opener.open(request).read()
                img_url = f[HSIZE:].split('<URL>')[1].split('</URL>')[0]
            except:
                print "except"
                pass

            if img_url:
                break
        
        if not img_url:
            return None

        pos = f.find('<LargeImage>', HSIZE+10) # FIXME, magic number
        if pos != -1:
            try:
                img_url = f[pos:].split('<URL>')[1].split('</URL>')[0]
            except IndexError:
                pass
        
        if img_url:
            urllib.urlretrieve(img_url, dest_filename)
            imgfound = dest_filename
        
        return imgfound

    def configure( self ):
        '''display a configuration dialog'''

        style = self.config.getPluginValue(self.name, 'style', \
            '%title - %artist (%album)')
        player = self.config.getPluginValue(self.name, 'player', 'Amarok')
        avatar = self.config.getPluginValue(self.name, 'avatar', '0') == '1'
        remotefetch = self.config.getPluginValue(self.name, \
            'remotefetch', '0') == '1'

        #self.updateCustomConf()
        l = []
        # name, optionType, label, description, value, options
        l.append(Plugin.Option('style', str, _('Display style'), \
            _('Change display style of the song you are listening, possible ' \
            'variables are %title, %artist and %album'), style))
        l.append(Plugin.Option('player', list, _('Music player'), \
            _('Select the music player that you are using'), player, \
            self.getPluginNames(), self.playerChanged))
        
        self.statusLabel = gtk.Label()
        status_timeout = gobject.timeout_add(2000, self.updateStatus)
        self.updateStatus()
        logsButton = gtk.Button( _('Show logs') )
        logsButton.connect('clicked', self.showLogs)
        hbox = gtk.HBox()
        hbox.pack_start(self.statusLabel, True, True)
        hbox.pack_start(logsButton)
        status = Plugin.Option('status', gtk.Widget, '', '', hbox)
        l.append(status)

        l.append(Plugin.Option('avatar', bool, _('Change Avatar'), \
            _('Change Avatar'), avatar))
        l.append(Plugin.Option( 'remotefetch', bool, \
            _( 'Get cover art from web' ), _( 'Get cover art from web' ), \
            remotefetch ))
        
        custom = Plugin.Option('custom', dict, _('Custom config'), \
            _('Custom config'), None, self.player.customConfig)
        l.append(custom)

        self.configWindow = Plugin.ConfigWindow( _( 'Config Plugin' ), l )
        self.customConfigWidget = custom.widget
        response = self.configWindow.run()

        if status_timeout:
            # this stops the update status timeout
            gobject.source_remove(status_timeout)

        if response != None:
            if response.has_key( 'player' ):
                self.setPlayer( response[ 'player' ].value )
                self.config.setPluginValue( self.name, 'player', \
                    response[ 'player' ].value )
                
            if response.has_key( 'style' ):
                self.player.setStyle( response[ 'style' ].value )
                self.config.setPluginValue( self.name, 'style', \
                    response['style'].value )
            if response.has_key('avatar'):
                self.avatar = response['avatar'].value
                self.config.setPluginValue(self.name, 'avatar', \
                    str(int(response['avatar'].value)))
            if response.has_key('remotefetch'):
                self.remotefetch =  response[ 'remotefetch' ].value 
                self.config.setPluginValue( self.name, 'remotefetch', \
                    str(int(response[ 'remotefetch' ].value)))
            if response.has_key('custom'):
                self.player.customConfig = self.customConfigWidget.data
                for key in self.player.customConfig.keys():
                    self.config.setPluginValue(self.name, \
                        str(self.playername + "_" + key), \
                        self.player.customConfig[key])
                self.player.updateConfig()
        self.configWindow = None
        self.statusLabel = None
        return True

    def updateStatus( self ):
        '''updates the status label'''
        if hasattr(self.player, 'status'):
            status = self.player.status
        else:
            status = 'unknown'

        if self.statusLabel:
            self.statusLabel.set_markup(_('Status:') + ' <b>%s</b>' % status)
            return True
        else:
            return False
        
    def showLogs( self, button ):
        '''Display an error log window'''
        log = []
        if hasattr(self.player, '_log'):
            log = self.player._log
        
        win = gtk.Dialog( _('Logs'), self.configWindow, \
            gtk.DIALOG_MODAL | gtk.DIALOG_DESTROY_WITH_PARENT, \
            (gtk.STOCK_OK, gtk.RESPONSE_ACCEPT) )
        win.set_default_size(300, 400)
        model = gtk.ListStore(str, str)
        tree = gtk.TreeView(model)
        crtkey = gtk.CellRendererText()
        crtvalue = gtk.CellRendererText()

        col1 = gtk.TreeViewColumn(_("Type"), crtkey, text=0)
        col2 = gtk.TreeViewColumn(_("Value"), crtvalue, text=1)
        tree.append_column(col1)
        tree.append_column(col2)
        for item in log:
            model.append(item)
        win.vbox.pack_start(tree)
        win.show_all()
        win.run()
        win.destroy()



# -*- coding: utf-8 -*-
#
#   This file is part of emesene.
#
#    Emesene is free software; you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation; either version 2 of the License, or
#    (at your option) any later version.
#
#    Emesene is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with emesene; if not, write to the Free Software
#    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
#

'''unit tests for pygif
requires netpbm tools (specifically "giftopnm")'''

import os
import unittest

import pygif

class CloneTests(unittest.TestCase):
    '''Tests that write a gif file based on an existing gif instance'''

    INPUT = 'vampire.gif'
    OUTPUT = 'unittest_clone.gif'
    
    def teststructure(self):
        '''structure: basic structure test, clones gif and encodes lzw'''
        gif = pygif.GifDecoder(open(self.INPUT, 'rb').read())

        enc = pygif.GifEncoder()
        enc.clone(gif)

        image = enc.new_image()
        image.codesize, image.lzwcode = enc.lzw_encode(gif.images[0].pixels)

        enc.write(self.OUTPUT)
        self.assert_(giftopnmtest(self.OUTPUT))

class RenderTests(unittest.TestCase):
    '''Tests that render gifs from scratch or user generated data'''
    
    INPUT = 'vampire.gif'
    OUTPUT = 'unittest_render.gif'
    
    def testgradient(self):
        '''gradient: outputs a 256 color greyscale gradient'''
        enc = pygif.GifEncoder()
        
        enc.header = 'GIF89a'
        enc.ls_width = 256
        enc.ls_height = 50
        
        enc.pallete = [(x, x, x) for x in range(256)]
        enc.global_color_table_size = len(enc.pallete)
        enc.color_table_flag = True
        enc.color_resolution = 7 # 256 colors
        enc.build_flags()
        
        pixels = [x for x in range(256)] * enc.ls_height

        image = enc.new_image()
        image.codesize, image.lzwcode = enc.lzw_encode(pixels)

        enc.write(self.OUTPUT)
        self.assert_(giftopnmtest(self.OUTPUT))


def giftopnmtest(path):
    '''runs giftopnm and returns True if it succeded'''
    return os.system("giftopnm " + path + " > /dev/null 2>&1") == 0

if __name__ == '__main__':
    unittest.main()



# -*- coding: utf-8 -*-

#   This file is part of emesene.
#
#    Emesene is free software; you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation; either version 2 of the License, or
#    (at your option) any later version.
#
#    emesene is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with emesene; if not, write to the Free Software
#    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA

import gtk
import emesenelib.common

class StatusMenu(gtk.Menu):
    '''This class represents the status menu where you can select a given
    status and it will be changed. This is a standalone class because
    it is used in two classes (MainMenu and TrayIcon)'''

    def __init__(self, controller):
        '''Contructor'''
        gtk.Menu.__init__(self)
    
        self.controller = controller
        self.theme = self.controller.theme

        j = 0
        for i in self.controller.status_ordered[0]:
            if i != "FLN":
                menuItem = self.newImageMenuItem (
                    self.controller.status_ordered[3][j], None,
                    self.controller.theme.statusToPixbuf(i))
                self.add(menuItem)
                menuItem.connect("activate", self.activate, i)
                j += 1

    def newImageMenuItem(self, label, stock = None, img = None):
        mi = gtk.ImageMenuItem(_(label))
            
        if stock != None:
            mi.set_image(gtk.image_new_from_stock(stock, gtk.ICON_SIZE_MENU))
        elif img != None:
            image = gtk.Image()
            image.set_from_pixbuf(img)
            mi.set_image(image)
        return mi

    def newCheckMenuItem(self, label, checked):
        mi = gtk.CheckMenuItem(_(label))
        mi.set_active(checked)
        return mi

    def activate(self, menuitem, status):
        '''change the status with the userparam'''
        self.controller.contacts.set_status(status)
        



# -*- coding: utf-8 -*-

#   This file is part of emesene.
#
#    Emesene is free software; you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation; either version 2 of the License, or
#    (at your option) any later version.
#
#    emesene is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with emesene; if not, write to the Free Software
#    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA

class SlashAction(object):
    def __init__(self, conversation, name, params=''):
        self.conversation = conversation
        self.name = name
        self.params = params
        
    def getConversation(self):
        return self.conversation
        
    def getParams(self):
        return self.params
        
    def outputText(self, string, send=False):
        if send:
            self.conversation.sendMessage(string)
        else:
            self.conversation.appendOutputText(None, string, 'error')

class SlashCommands(object):
    def __init__(self, controller):
        self.commands = {}
        self.register('help', self.slashHelp)
        self.controller = controller
        self.sendmessageID = controller.conversationManager.connect(
            'send-message', self.sendMessage)
    
    def unregister_slash(self):
        self.controller.conversationManager.disconnect(self.sendmessageID)
        
    def sendMessage(self, obj, conversation, message):
        '''Send Message Interceptor'''
        if len(message) < 3:
            return
        
        if message[0] == '/':
            #if the message start character is a slash,
            #we stop the message sending process and emit a signal
            obj.emit_stop_by_name('send-message')
            if not message[1] == '/':
                emit(conversation, self.commands, message)
            else:
                conversation.do_send_message(message[1:])
        
    def register(self, command, callback, help=None):
        if not self.commands.has_key(command):
            self.commands[command] = [callback, help]
        else:
            print 'The command', command,'is registered, try another'
        
    def unregister(self, command):
        if self.commands.has_key(command):
            self.commands.pop(command)
        
    def getCommandHelp(self, command):
        if self.commands.has_key(command):
            if self.commands[command][1] != None:
                return self.commands[command][1]
            else:
                return 'The command %s has not help' % command 
        else:
            return 'The command %s doesn\'t exist' % command
        
    def slashHelp(self, slashAction):
        params = slashAction.getParams()
        if params:
            slashAction.outputText(self.getCommandHelp(params))
        else:
            text = [_('Use "/help <command>" for help on a specific command'),
                    _('The following commands are available:'),
                    ', '.join(self.commands.keys())]
            slashAction.outputText('\n'.join(text))

def emit(conversation, commands, message):
    name = params = ''
    cmd = message[1:].split(" ",1)

    name = cmd[0]
    if len(cmd) > 1:
        params = cmd[1]

    if commands.has_key(name):
        slashAction = SlashAction(conversation, name, params)
        commands[name][0](slashAction)
    else:
        conversation.appendOutputText(None,
            _('The command %s doesn\'t exist') % name, 'error')
